@@ -36,16 +36,17 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-// NOTE: alphabetically ordered
-#include "nsAccessibleTreeWalker.h"
-#include "nsAccessibilityAtoms.h"
 #include "nsHTMLFormControlAccessible.h"
+
+#include "nsAccessibilityAtoms.h"
+#include "nsAccUtils.h"
+#include "nsRelUtils.h"
+#include "nsTextEquivUtils.h"
+
 #include "nsIDOMDocument.h"
-#include "nsIDOMNSHTMLInputElement.h"
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMNSHTMLElement.h"
 #include "nsIDOMNSEditableElement.h"
-#include "nsIDOMNSHTMLButtonElement.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMHTMLLegendElement.h"
 #include "nsIDOMHTMLTextAreaElement.h"
@@ -58,18 +59,20 @@
 #include "nsIServiceManager.h"
 #include "nsITextControlFrame.h"
 
-// --- checkbox -----
+////////////////////////////////////////////////////////////////////////////////
+// nsHTMLCheckboxAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsHTMLCheckboxAccessible::nsHTMLCheckboxAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsFormControlAccessible(aNode, aShell)
-{ 
+nsHTMLCheckboxAccessible::
+  nsHTMLCheckboxAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsFormControlAccessible(aContent, aShell)
+{
 }
 
-nsresult
-nsHTMLCheckboxAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsHTMLCheckboxAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_CHECKBUTTON;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_CHECKBUTTON;
 }
 
 NS_IMETHODIMP nsHTMLCheckboxAccessible::GetNumActions(PRUint8 *_retval)
@@ -98,12 +101,14 @@ NS_IMETHODIMP nsHTMLCheckboxAccessible::GetActionName(PRUint8 aIndex, nsAString&
   return NS_ERROR_INVALID_ARG;
 }
 
-NS_IMETHODIMP nsHTMLCheckboxAccessible::DoAction(PRUint8 index)
+NS_IMETHODIMP
+nsHTMLCheckboxAccessible::DoAction(PRUint8 aIndex)
 {
-  if (index == 0) {   // 0 is the magic value for default action
-    return DoCommand();
-  }
-  return NS_ERROR_INVALID_ARG;
+  if (aIndex != 0)
+    return NS_ERROR_INVALID_ARG;
+
+  DoCommand();
+  return NS_OK;
 }
 
 nsresult
@@ -115,37 +120,34 @@ nsHTMLCheckboxAccessible::GetStateInternal(PRUint32 *aState,
 
   *aState |= nsIAccessibleStates::STATE_CHECKABLE;
 
-  PRBool checked = PR_FALSE;   // Radio buttons and check boxes can be checked
-  PRBool mixed = PR_FALSE;     // or mixed.
+  PRBool state = PR_FALSE;   // Radio buttons and check boxes can be checked or mixed
 
-  nsCOMPtr<nsIDOMNSHTMLInputElement>
-           html5CheckboxElement(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMHTMLInputElement> htmlCheckboxElement =
+    do_QueryInterface(mContent);
            
-  if (html5CheckboxElement) {
-    html5CheckboxElement->GetIndeterminate(&mixed);
+  if (htmlCheckboxElement) {
+    htmlCheckboxElement->GetIndeterminate(&state);
 
-    if (mixed) {
+    if (state) {
       *aState |= nsIAccessibleStates::STATE_MIXED;
-      return NS_OK;  // indeterminate can't be checked at the same time.
+    } else {   // indeterminate can't be checked at the same time.
+      htmlCheckboxElement->GetChecked(&state);
+    
+      if (state)
+        *aState |= nsIAccessibleStates::STATE_CHECKED;
     }
   }
-  
-  nsCOMPtr<nsIDOMHTMLInputElement> htmlCheckboxElement(do_QueryInterface(mDOMNode));
-  if (htmlCheckboxElement) {
-    htmlCheckboxElement->GetChecked(&checked);
-  
-    if (checked)
-      *aState |= nsIAccessibleStates::STATE_CHECKED;
-  }
-
   return NS_OK;
 }
 
-//------ Radio button -------
+////////////////////////////////////////////////////////////////////////////////
+// nsHTMLRadioButtonAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsHTMLRadioButtonAccessible::nsHTMLRadioButtonAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsRadioButtonAccessible(aNode, aShell)
-{ 
+nsHTMLRadioButtonAccessible::
+  nsHTMLRadioButtonAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsRadioButtonAccessible(aContent, aShell)
+{
 }
 
 nsresult
@@ -159,7 +161,8 @@ nsHTMLRadioButtonAccessible::GetStateInternal(PRUint32 *aState,
   
   PRBool checked = PR_FALSE;   // Radio buttons and check boxes can be checked
 
-  nsCOMPtr<nsIDOMHTMLInputElement> htmlRadioElement(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMHTMLInputElement> htmlRadioElement =
+    do_QueryInterface(mContent);
   if (htmlRadioElement)
     htmlRadioElement->GetChecked(&checked);
 
@@ -169,48 +172,40 @@ nsHTMLRadioButtonAccessible::GetStateInternal(PRUint32 *aState,
   return NS_OK;
 }
 
-nsresult
-nsHTMLRadioButtonAccessible::GetAttributesInternal(nsIPersistentProperties *aAttributes)
+void
+nsHTMLRadioButtonAccessible::GetPositionAndSizeInternal(PRInt32 *aPosInSet,
+                                                        PRInt32 *aSetSize)
 {
-  NS_ENSURE_ARG_POINTER(aAttributes);
-  NS_ENSURE_TRUE(mDOMNode, NS_ERROR_FAILURE);
-
-  nsresult rv = nsRadioButtonAccessible::GetAttributesInternal(aAttributes);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsAutoString nsURI;
-  mDOMNode->GetNamespaceURI(nsURI);
+  mContent->NodeInfo()->GetNamespaceURI(nsURI);
   nsAutoString tagName;
-  mDOMNode->GetLocalName(tagName);
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-  NS_ENSURE_STATE(content);
+  mContent->NodeInfo()->GetName(tagName);
 
   nsAutoString type;
-  content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::type, type);
+  mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::type, type);
   nsAutoString name;
-  content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::name, name);
+  mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::name, name);
 
   nsCOMPtr<nsIDOMNodeList> inputs;
 
-  nsCOMPtr<nsIDOMHTMLInputElement> radio(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMHTMLInputElement> radio(do_QueryInterface(mContent));
   nsCOMPtr<nsIDOMHTMLFormElement> form;
   radio->GetForm(getter_AddRefs(form));
   if (form) {
     form->GetElementsByTagNameNS(nsURI, tagName, getter_AddRefs(inputs));
   } else {
-    nsCOMPtr<nsIDOMDocument> document;
-    mDOMNode->GetOwnerDocument(getter_AddRefs(document));
+    nsIDocument* doc = mContent->GetOwnerDoc();
+    nsCOMPtr<nsIDOMDocument> document(do_QueryInterface(doc));
     if (document)
       document->GetElementsByTagNameNS(nsURI, tagName, getter_AddRefs(inputs));
   }
 
-  NS_ENSURE_TRUE(inputs, NS_OK);
+  NS_ENSURE_TRUE(inputs, );
 
   PRUint32 inputsCount = 0;
   inputs->GetLength(&inputsCount);
 
-  // Get posinset and setsize.
+  // Compute posinset and setsize.
   PRInt32 indexOf = 0;
   PRInt32 count = 0;
 
@@ -227,21 +222,23 @@ nsHTMLRadioButtonAccessible::GetAttributesInternal(nsIPersistentProperties *aAtt
 
       count++;
 
-      if (itemNode == mDOMNode)
+      if (item == mContent)
         indexOf = count;
     }
   }
 
-  nsAccUtils::SetAccGroupAttrs(aAttributes, 0, indexOf, count);
-
-  return  NS_OK;
+  *aPosInSet = indexOf;
+  *aSetSize = count;
 }
 
-// ----- Button -----
+////////////////////////////////////////////////////////////////////////////////
+// nsHTMLButtonAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsHTMLButtonAccessible::nsHTMLButtonAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsHyperTextAccessibleWrap(aNode, aShell)
-{ 
+nsHTMLButtonAccessible::
+  nsHTMLButtonAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
+{
 }
 
 NS_IMETHODIMP nsHTMLButtonAccessible::GetNumActions(PRUint8 *_retval)
@@ -259,12 +256,14 @@ NS_IMETHODIMP nsHTMLButtonAccessible::GetActionName(PRUint8 aIndex, nsAString& a
   return NS_ERROR_INVALID_ARG;
 }
 
-NS_IMETHODIMP nsHTMLButtonAccessible::DoAction(PRUint8 index)
+NS_IMETHODIMP
+nsHTMLButtonAccessible::DoAction(PRUint8 aIndex)
 {
-  if (index == eAction_Click) {
-    return DoCommand();
-  }
-  return NS_ERROR_INVALID_ARG;
+  if (aIndex != eAction_Click)
+    return NS_ERROR_INVALID_ARG;
+
+  DoCommand();
+  return NS_OK;
 }
 
 nsresult
@@ -275,22 +274,17 @@ nsHTMLButtonAccessible::GetStateInternal(PRUint32 *aState,
                                                             aExtraState);
   NS_ENSURE_A11Y_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMElement> element(do_QueryInterface(mDOMNode));
-  NS_ENSURE_TRUE(element, NS_ERROR_FAILURE);
-
-  nsAutoString buttonType;
-  element->GetAttribute(NS_LITERAL_STRING("type"), buttonType);
-  if (buttonType.LowerCaseEqualsLiteral("submit"))
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
+                            nsAccessibilityAtoms::submit, eIgnoreCase))
     *aState |= nsIAccessibleStates::STATE_DEFAULT;
 
   return NS_OK;
 }
 
-nsresult
-nsHTMLButtonAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsHTMLButtonAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_PUSHBUTTON;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_PUSHBUTTON;
 }
 
 nsresult
@@ -300,14 +294,12 @@ nsHTMLButtonAccessible::GetNameInternal(nsAString& aName)
   if (!aName.IsEmpty())
     return NS_OK;
 
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-
   // No name from HTML or ARIA
   nsAutoString name;
-  if (!content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::value,
-                        name) &&
-      !content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::alt,
-                        name)) {
+  if (!mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::value,
+                         name) &&
+      !mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::alt,
+                         name)) {
     // Use the button's (default) label if nothing else works
     nsIFrame* frame = GetFrame();
     if (frame) {
@@ -318,9 +310,9 @@ nsHTMLButtonAccessible::GetNameInternal(nsAString& aName)
   }
 
   if (name.IsEmpty() &&
-      !content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::src,
-                        name)) {
-    content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::data, name);
+      !mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::src,
+                         name)) {
+    mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::data, name);
   }
 
   name.CompressWhitespace();
@@ -330,11 +322,14 @@ nsHTMLButtonAccessible::GetNameInternal(nsAString& aName)
 }
 
 
-// ----- HTML 4 Button: can contain arbitrary HTML content -----
+////////////////////////////////////////////////////////////////////////////////
+// nsHTML4ButtonAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsHTML4ButtonAccessible::nsHTML4ButtonAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsHyperTextAccessibleWrap(aNode, aShell)
-{ 
+nsHTML4ButtonAccessible::
+  nsHTML4ButtonAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
+{
 }
 
 NS_IMETHODIMP nsHTML4ButtonAccessible::GetNumActions(PRUint8 *_retval)
@@ -352,19 +347,20 @@ NS_IMETHODIMP nsHTML4ButtonAccessible::GetActionName(PRUint8 aIndex, nsAString&
   return NS_ERROR_INVALID_ARG;
 }
 
-NS_IMETHODIMP nsHTML4ButtonAccessible::DoAction(PRUint8 index)
+NS_IMETHODIMP
+nsHTML4ButtonAccessible::DoAction(PRUint8 aIndex)
 {
-  if (index == 0) {
-    return DoCommand();
-  }
-  return NS_ERROR_INVALID_ARG;
+  if (aIndex != 0)
+    return NS_ERROR_INVALID_ARG;
+
+  DoCommand();
+  return NS_OK;
 }
 
-nsresult
-nsHTML4ButtonAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsHTML4ButtonAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_PUSHBUTTON;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_PUSHBUTTON;
 }
 
 nsresult
@@ -375,39 +371,36 @@ nsHTML4ButtonAccessible::GetStateInternal(PRUint32 *aState,
                                                             aExtraState);
   NS_ENSURE_A11Y_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMElement> element(do_QueryInterface(mDOMNode));
-  NS_ASSERTION(element, "No element for button's dom node!");
-
   *aState |= nsIAccessibleStates::STATE_FOCUSABLE;
 
-  nsAutoString buttonType;
-  element->GetAttribute(NS_LITERAL_STRING("type"), buttonType);
-  if (buttonType.LowerCaseEqualsLiteral("submit"))
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
+                            nsAccessibilityAtoms::submit, eIgnoreCase))
     *aState |= nsIAccessibleStates::STATE_DEFAULT;
 
   return NS_OK;
 }
 
-// --- textfield -----
 
-nsHTMLTextFieldAccessible::nsHTMLTextFieldAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsHyperTextAccessibleWrap(aNode, aShell)
+////////////////////////////////////////////////////////////////////////////////
+// nsHTMLTextFieldAccessible
+////////////////////////////////////////////////////////////////////////////////
+
+nsHTMLTextFieldAccessible::
+  nsHTMLTextFieldAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
 {
 }
 
 NS_IMPL_ISUPPORTS_INHERITED3(nsHTMLTextFieldAccessible, nsAccessible, nsHyperTextAccessible, nsIAccessibleText, nsIAccessibleEditableText)
 
-nsresult
-nsHTMLTextFieldAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsHTMLTextFieldAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_ENTRY;
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-  if (content &&
-      content->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
-                           nsAccessibilityAtoms::password, eIgnoreCase)) {
-    *aRole = nsIAccessibleRole::ROLE_PASSWORD_TEXT;
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
+                            nsAccessibilityAtoms::password, eIgnoreCase)) {
+    return nsIAccessibleRole::ROLE_PASSWORD_TEXT;
   }
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_ENTRY;
 }
 
 nsresult
@@ -419,18 +412,24 @@ nsHTMLTextFieldAccessible::GetNameInternal(nsAString& aName)
   if (!aName.IsEmpty())
     return NS_OK;
 
-  nsCOMPtr<nsIContent> content = do_QueryInterface(mDOMNode);
-  if (!content->GetBindingParent())
+  if (mContent->GetBindingParent())
+  {
+    // XXX: bug 459640
+    // There's a binding parent.
+    // This means we're part of another control, so use parent accessible for name.
+    // This ensures that a textbox inside of a XUL widget gets
+    // an accessible name.
+    nsAccessible* parent = GetParent();
+    parent->GetName(aName);
+  }
+
+  if (!aName.IsEmpty())
     return NS_OK;
 
-  // XXX: bug 459640
-  // There's a binding parent.
-  // This means we're part of another control, so use parent accessible for name.
-  // This ensures that a textbox inside of a XUL widget gets
-  // an accessible name.
-  nsCOMPtr<nsIAccessible> parent;
-  rv = GetParent(getter_AddRefs(parent));
-  return parent ? parent->GetName(aName) : rv;
+  // text inputs and textareas might have useful placeholder text
+  mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::placeholder, aName);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsHTMLTextFieldAccessible::GetValue(nsAString& _retval)
@@ -442,12 +441,12 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetValue(nsAString& _retval)
   if (state & nsIAccessibleStates::STATE_PROTECTED)    // Don't return password text!
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIDOMHTMLTextAreaElement> textArea(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMHTMLTextAreaElement> textArea(do_QueryInterface(mContent));
   if (textArea) {
     return textArea->GetValue(_retval);
   }
   
-  nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(mContent));
   if (inputElement) {
     return inputElement->GetValue(_retval);
   }
@@ -464,28 +463,24 @@ nsHTMLTextFieldAccessible::GetStateInternal(PRUint32 *aState,
   NS_ENSURE_A11Y_SUCCESS(rv, rv);
 
   // can be focusable, focused, protected. readonly, unavailable, selected
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-  NS_ASSERTION(content, "Should not have gotten here if upcalled GetExtState() succeeded");
-
-  if (content->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
-                           nsAccessibilityAtoms::password, eIgnoreCase)) {
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
+                            nsAccessibilityAtoms::password, eIgnoreCase)) {
     *aState |= nsIAccessibleStates::STATE_PROTECTED;
   }
   else {
-    nsCOMPtr<nsIAccessible> parent;
-    GetParent(getter_AddRefs(parent));
-    if (nsAccUtils::Role(parent) == nsIAccessibleRole::ROLE_AUTOCOMPLETE)
+    nsAccessible* parent = GetParent();
+    if (parent && parent->Role() == nsIAccessibleRole::ROLE_AUTOCOMPLETE)
       *aState |= nsIAccessibleStates::STATE_HASPOPUP;
   }
 
-  if (content->HasAttr(kNameSpaceID_None, nsAccessibilityAtoms::readonly)) {
+  if (mContent->HasAttr(kNameSpaceID_None, nsAccessibilityAtoms::readonly)) {
     *aState |= nsIAccessibleStates::STATE_READONLY;
   }
 
   if (!aExtraState)
     return NS_OK;
 
-  nsCOMPtr<nsIDOMHTMLInputElement> htmlInput(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMHTMLInputElement> htmlInput(do_QueryInterface(mContent));
   // Is it an <input> or a <textarea> ?
   if (htmlInput) {
     *aExtraState |= nsIAccessibleStates::EXT_STATE_SINGLE_LINE;
@@ -497,16 +492,17 @@ nsHTMLTextFieldAccessible::GetStateInternal(PRUint32 *aState,
   if (!(*aExtraState & nsIAccessibleStates::EXT_STATE_EDITABLE))
     return NS_OK;
 
-  nsCOMPtr<nsIContent> bindingContent = content->GetBindingParent();
+  nsCOMPtr<nsIContent> bindingContent = mContent->GetBindingParent();
   if (bindingContent &&
       bindingContent->NodeInfo()->Equals(nsAccessibilityAtoms::textbox,
-                                         kNameSpaceID_XUL) &&
-      bindingContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
-                                  nsAccessibilityAtoms::autocomplete,
-                                  eIgnoreCase)) {
-    // If parent is XUL textbox and value of @type attribute is "autocomplete",
-    // then this accessible supports autocompletion.
-    *aExtraState |= nsIAccessibleStates::EXT_STATE_SUPPORTS_AUTOCOMPLETION;
+                                         kNameSpaceID_XUL)) {
+     if (bindingContent->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
+                                     nsAccessibilityAtoms::autocomplete,
+                                     eIgnoreCase)) {
+       // If parent is XUL textbox and value of @type attribute is "autocomplete",
+       // then this accessible supports autocompletion.
+       *aExtraState |= nsIAccessibleStates::EXT_STATE_SUPPORTS_AUTOCOMPLETION;
+     }
   } else if (gIsFormFillEnabled && htmlInput &&
              !(*aState & nsIAccessibleStates::STATE_PROTECTED)) {
     // Check to see if autocompletion is allowed on this input. We don't expose
@@ -515,8 +511,8 @@ nsHTMLTextFieldAccessible::GetStateInternal(PRUint32 *aState,
     // we're talking here is based on what the user types, where a popup of
     // possible choices comes up.
     nsAutoString autocomplete;
-    content->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::autocomplete,
-                     autocomplete);
+    mContent->GetAttr(kNameSpaceID_None, nsAccessibilityAtoms::autocomplete,
+                      autocomplete);
 
     if (!autocomplete.LowerCaseEqualsLiteral("off")) {
       nsCOMPtr<nsIDOMHTMLFormElement> form;
@@ -553,7 +549,7 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetActionName(PRUint8 aIndex, nsAString
 NS_IMETHODIMP nsHTMLTextFieldAccessible::DoAction(PRUint8 index)
 {
   if (index == 0) {
-    nsCOMPtr<nsIDOMNSHTMLElement> element(do_QueryInterface(mDOMNode));
+    nsCOMPtr<nsIDOMHTMLElement> element(do_QueryInterface(mContent));
     if ( element ) {
       return element->Focus();
     }
@@ -565,7 +561,7 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::DoAction(PRUint8 index)
 NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAssociatedEditor(nsIEditor **aEditor)
 {
   *aEditor = nsnull;
-  nsCOMPtr<nsIDOMNSEditableElement> editableElt(do_QueryInterface(mDOMNode));
+  nsCOMPtr<nsIDOMNSEditableElement> editableElt(do_QueryInterface(mContent));
   NS_ENSURE_TRUE(editableElt, NS_ERROR_FAILURE);
 
   // nsGenericHTMLElement::GetEditor has a security check.
@@ -587,36 +583,31 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAssociatedEditor(nsIEditor **aEditor
   return rv;
 }
 
-// --- groupbox  -----
+////////////////////////////////////////////////////////////////////////////////
+// nsHTMLGroupboxAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-/*
- * The HTML for this is <fieldset> <legend>box-title</legend> form elements </fieldset> 
- */
-
-nsHTMLGroupboxAccessible::nsHTMLGroupboxAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsHyperTextAccessibleWrap(aNode, aShell)
-{ 
+nsHTMLGroupboxAccessible::
+  nsHTMLGroupboxAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
+{
 }
 
-nsresult
-nsHTMLGroupboxAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsHTMLGroupboxAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_GROUPING;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_GROUPING;
 }
 
 nsIContent* nsHTMLGroupboxAccessible::GetLegend()
 {
-  nsCOMPtr<nsIContent> content = do_QueryInterface(mDOMNode);
-  NS_ENSURE_TRUE(content, nsnull);
-
   nsresult count = 0;
-  nsIContent *testLegendContent;
-  while ((testLegendContent = content->GetChildAt(count ++ )) != nsnull) {
-    if (testLegendContent->NodeInfo()->Equals(nsAccessibilityAtoms::legend,
-                                              content->GetNameSpaceID())) {
+  nsIContent *legendContent = nsnull;
+  while ((legendContent = mContent->GetChildAt(count++)) != nsnull) {
+    if (legendContent->NodeInfo()->Equals(nsAccessibilityAtoms::legend,
+                                          mContent->GetNameSpaceID())) {
       // Either XHTML namespace or no namespace
-      return testLegendContent;
+      return legendContent;
     }
   }
 
@@ -658,12 +649,15 @@ nsHTMLGroupboxAccessible::GetRelationByType(PRUint32 aRelationType,
   return NS_OK;
 }
 
+
 ////////////////////////////////////////////////////////////////////////////////
 // nsHTMLLegendAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsHTMLLegendAccessible::nsHTMLLegendAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsHyperTextAccessibleWrap(aNode, aShell)
-{ 
+nsHTMLLegendAccessible::
+  nsHTMLLegendAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
+{
 }
 
 NS_IMETHODIMP
@@ -676,19 +670,20 @@ nsHTMLLegendAccessible::GetRelationByType(PRUint32 aRelationType,
 
   if (aRelationType == nsIAccessibleRelation::RELATION_LABEL_FOR) {
     // Look for groupbox parent
-    nsCOMPtr<nsIAccessible> groupboxAccessible = GetParent();
-    if (nsAccUtils::Role(groupboxAccessible) == nsIAccessibleRole::ROLE_GROUPING) {
+    nsAccessible* groupbox = GetParent();
+
+    if (groupbox && groupbox->Role() == nsIAccessibleRole::ROLE_GROUPING) {
       // XXX: if group box exposes more than one relation of the given type
       // then we fail.
       nsCOMPtr<nsIAccessible> testLabelAccessible =
-        nsRelUtils::GetRelatedAccessible(groupboxAccessible,
+        nsRelUtils::GetRelatedAccessible(groupbox,
                                          nsIAccessibleRelation::RELATION_LABELLED_BY);
 
       if (testLabelAccessible == this) {
         // We're the first child of the parent groupbox, see
         // nsHTMLGroupboxAccessible::GetRelationByType().
         return nsRelUtils::
-          AddTarget(aRelationType, aRelation, groupboxAccessible);
+          AddTarget(aRelationType, aRelation, groupbox);
       }
     }
   }
@@ -696,9 +691,8 @@ nsHTMLLegendAccessible::GetRelationByType(PRUint32 aRelationType,
   return NS_OK;
 }
 
-nsresult
-nsHTMLLegendAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsHTMLLegendAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_LABEL;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_LABEL;
 }
