@@ -36,9 +36,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsAccessibleWrap.h"
-#include "nsIAccessibleDocument.h"
-#include "nsIAccessibleText.h"
+#include "nsDocAccessible.h"
 #include "nsObjCExceptions.h"
 
 #import "nsRoleMap.h"
@@ -48,9 +46,9 @@
 #import "mozActionElements.h"
 #import "mozTextAccessible.h"
 
-nsAccessibleWrap::nsAccessibleWrap(nsIDOMNode* aNode, nsIWeakReference *aShell): 
-  nsAccessible(aNode, aShell),
-  mNativeWrapper(nsnull)
+nsAccessibleWrap::
+  nsAccessibleWrap(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsAccessible(aContent, aShell), mNativeWrapper(nsnull)
 {
 }
 
@@ -62,19 +60,20 @@ nsAccessibleWrap::~nsAccessibleWrap()
   }
 }
 
-nsresult
+PRBool
 nsAccessibleWrap::Init () 
 {
-  // need to pass the call up, so we're cached (which nsAccessNode::Init() takes care of).
-  nsresult rv = nsAccessible::Init();
-  NS_ENSURE_SUCCESS(rv, rv);
-  
+  if (!nsAccessible::Init())
+    return PR_FALSE;
+
   if (!mNativeWrapper && !AncestorIsFlat()) {
     // Create our native object using the class type specified in GetNativeType().
     mNativeWrapper = new AccessibleWrapper (this, GetNativeType());
+    if (!mNativeWrapper)
+      return PR_FALSE;
   }
 
-  return NS_OK;
+  return PR_TRUE;
 }
 
 NS_IMETHODIMP
@@ -87,15 +86,6 @@ nsAccessibleWrap::GetNativeInterface (void **aOutInterface)
   return NS_ERROR_FAILURE;
 }
 
-// get the native NSWindow we reside in.
-void
-nsAccessibleWrap::GetNativeWindow (void **aOutNativeWindow)
-{
-  *aOutNativeWindow = nsnull;
-  nsCOMPtr<nsIAccessibleDocument> docAccessible(GetDocAccessible());
-  docAccessible->GetWindowHandle (aOutNativeWindow);
-}
-
 // overridden in subclasses to create the right kind of object. by default we create a generic
 // 'mozAccessible' node.
 objc_class*
@@ -103,7 +93,7 @@ nsAccessibleWrap::GetNativeType ()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
 
-  PRUint32 role = nsAccUtils::Role(this);
+  PRUint32 role = Role();
   switch (role) {
     case nsIAccessibleRole::ROLE_PUSHBUTTON:
     case nsIAccessibleRole::ROLE_SPLITBUTTON:
@@ -148,7 +138,7 @@ nsAccessibleWrap::GetNativeType ()
 // this method is very important. it is fired when an accessible object "dies". after this point
 // the object might still be around (because some 3rd party still has a ref to it), but it is
 // in fact 'dead'.
-nsresult
+void
 nsAccessibleWrap::Shutdown ()
 {
   if (mNativeWrapper) {
@@ -156,17 +146,15 @@ nsAccessibleWrap::Shutdown ()
     mNativeWrapper = nsnull;
   }
   
-  return nsAccessible::Shutdown();
+  nsAccessible::Shutdown();
 }
 
 nsresult
-nsAccessibleWrap::FireAccessibleEvent(nsIAccessibleEvent *aEvent)
+nsAccessibleWrap::HandleAccEvent(AccEvent* aEvent)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
-  NS_ENSURE_ARG_POINTER(aEvent);
-
-  nsresult rv = nsAccessible::FireAccessibleEvent(aEvent);
+  nsresult rv = nsAccessible::HandleAccEvent(aEvent);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return FirePlatformEvent(aEvent);
@@ -175,21 +163,18 @@ nsAccessibleWrap::FireAccessibleEvent(nsIAccessibleEvent *aEvent)
 }
 
 nsresult
-nsAccessibleWrap::FirePlatformEvent(nsIAccessibleEvent *aEvent)
+nsAccessibleWrap::FirePlatformEvent(AccEvent* aEvent)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
-  PRUint32 eventType;
-  nsresult rv = aEvent->GetEventType(&eventType);
-  NS_ENSURE_SUCCESS(rv, rv);
+  PRUint32 eventType = aEvent->GetEventType();
 
   // ignore everything but focus-changed and value-changed events for now.
   if (eventType != nsIAccessibleEvent::EVENT_FOCUS &&
       eventType != nsIAccessibleEvent::EVENT_VALUE_CHANGE)
     return NS_OK;
 
-  nsCOMPtr<nsIAccessible> accessible;
-  rv = aEvent->GetAccessible(getter_AddRefs(accessible));
+  nsAccessible *accessible = aEvent->GetAccessible();
   NS_ENSURE_STATE(accessible);
 
   mozAccessible *nativeAcc = nil;
@@ -231,35 +216,34 @@ nsAccessibleWrap::GetUnignoredChildCount(PRBool aDeepCount)
   // if we're flat, we have no children.
   if (nsAccUtils::MustPrune(this))
     return 0;
-  
-  PRInt32 childCount = 0;
-  GetChildCount(&childCount);
-  
-  nsCOMPtr<nsIAccessible> curAcc;
-  
-  while (NextChild(curAcc)) {
-    nsAccessibleWrap *childWrap = static_cast<nsAccessibleWrap*>(curAcc.get());
-    
+
+  PRInt32 resultChildCount = 0;
+
+  PRInt32 childCount = GetChildCount();
+  for (PRInt32 childIdx = 0; childIdx < childCount; childIdx++) {
+    nsAccessibleWrap *childAcc =
+      static_cast<nsAccessibleWrap*>(GetChildAt(childIdx));
+
     // if the current child is not ignored, count it.
-    if (!childWrap->IsIgnored())
-      ++childCount;
-      
+    if (!childAcc->IsIgnored())
+      ++resultChildCount;
+
     // if it's flat, we don't care to inspect its children.
-    if (nsAccUtils::MustPrune(childWrap))
+    if (nsAccUtils::MustPrune(childAcc))
       continue;
-    
+
     if (aDeepCount) {
       // recursively count the unignored children of our children since it's a deep count.
-      childCount += childWrap->GetUnignoredChildCount(PR_TRUE);
+      resultChildCount += childAcc->GetUnignoredChildCount(PR_TRUE);
     } else {
       // no deep counting, but if the child is ignored, we want to substitute it for its
       // children.
-      if (childWrap->IsIgnored()) 
-        childCount += childWrap->GetUnignoredChildCount(PR_FALSE);
+      if (childAcc->IsIgnored()) 
+        resultChildCount += childAcc->GetUnignoredChildCount(PR_FALSE);
     }
   } 
   
-  return childCount;
+  return resultChildCount;
 }
 
 // if we for some reason have no native accessible, we should be skipped over (and traversed)
@@ -273,19 +257,20 @@ nsAccessibleWrap::IsIgnored()
 void
 nsAccessibleWrap::GetUnignoredChildren(nsTArray<nsRefPtr<nsAccessibleWrap> > &aChildrenArray)
 {
-  nsCOMPtr<nsIAccessible> curAcc;
-  
   // we're flat; there are no children.
   if (nsAccUtils::MustPrune(this))
     return;
-  
-  while (NextChild(curAcc)) {
-    nsAccessibleWrap *childWrap = static_cast<nsAccessibleWrap*>(curAcc.get());
-    if (childWrap->IsIgnored()) {
+
+  PRInt32 childCount = GetChildCount();
+  for (PRInt32 childIdx = 0; childIdx < childCount; childIdx++) {
+    nsAccessibleWrap *childAcc =
+      static_cast<nsAccessibleWrap*>(GetChildAt(childIdx));
+
+    if (childAcc->IsIgnored()) {
       // element is ignored, so try adding its children as substitutes, if it has any.
-      if (!nsAccUtils::MustPrune(childWrap)) {
+      if (!nsAccUtils::MustPrune(childAcc)) {
         nsTArray<nsRefPtr<nsAccessibleWrap> > children;
-        childWrap->GetUnignoredChildren(children);
+        childAcc->GetUnignoredChildren(children);
         if (!children.IsEmpty()) {
           // add the found unignored descendants to the array.
           aChildrenArray.AppendElements(children);
@@ -293,15 +278,14 @@ nsAccessibleWrap::GetUnignoredChildren(nsTArray<nsRefPtr<nsAccessibleWrap> > &aC
       }
     } else
       // simply add the element, since it's not ignored.
-      aChildrenArray.AppendElement(childWrap);
+      aChildrenArray.AppendElement(childAcc);
   }
 }
 
 already_AddRefed<nsIAccessible>
 nsAccessibleWrap::GetUnignoredParent()
 {
-  nsCOMPtr<nsIAccessible> parent(GetParent());
-  nsAccessibleWrap *parentWrap = static_cast<nsAccessibleWrap*>(parent.get());
+  nsAccessibleWrap *parentWrap = static_cast<nsAccessibleWrap*>(GetParent());
   if (!parentWrap)
     return nsnull;
     
@@ -310,7 +294,32 @@ nsAccessibleWrap::GetUnignoredParent()
     return parentWrap->GetUnignoredParent();
   
   nsIAccessible *outValue = nsnull;
-  NS_IF_ADDREF(outValue = parent.get());
+  NS_IF_ADDREF(outValue = parentWrap);
   
   return outValue;
 }
+
+////////////////////////////////////////////////////////////////////////////////
+// nsAccessibleWrap protected
+
+PRBool
+nsAccessibleWrap::AncestorIsFlat()
+{
+  // We don't create a native object if we're child of a "flat" accessible;
+  // for example, on OS X buttons shouldn't have any children, because that
+  // makes the OS confused. 
+  //
+  // To maintain a scripting environment where the XPCOM accessible hierarchy
+  // look the same on all platforms, we still let the C++ objects be created
+  // though.
+
+  nsAccessible* parent(GetParent());
+  while (parent) {
+    if (nsAccUtils::MustPrune(parent))
+      return PR_TRUE;
+
+    parent = parent->GetParent();
+  }
+  // no parent was flat
+  return PR_FALSE;
+}
