@@ -61,6 +61,7 @@
 #include "nsLayoutUtils.h"
 #include "nsISelection2.h"
 #include "nsIMEStateManager.h"
+#include "nsIObjectFrame.h"
 
 nsresult NS_NewContentIterator(nsIContentIterator** aInstancePtrResult);
 
@@ -77,36 +78,25 @@ nsContentEventHandler::nsContentEventHandler(
 }
 
 nsresult
-nsContentEventHandler::Init(nsQueryContentEvent* aEvent)
+nsContentEventHandler::InitCommon()
 {
-  NS_ASSERTION(aEvent, "aEvent must not be null");
-
   if (mSelection)
     return NS_OK;
 
-  aEvent->mSucceeded = PR_FALSE;
-
-  if (!mPresShell)
-    return NS_ERROR_NOT_AVAILABLE;
+  NS_ENSURE_TRUE(mPresShell, NS_ERROR_NOT_AVAILABLE);
 
   // If text frame which has overflowing selection underline is dirty,
   // we need to flush the pending reflow here.
-  nsresult rv = mPresShell->FlushPendingNotifications(Flush_Layout);
-  NS_ENSURE_SUCCESS(rv, rv);
+  mPresShell->FlushPendingNotifications(Flush_Layout);
 
-  rv = mPresShell->GetSelectionForCopy(getter_AddRefs(mSelection));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(mSelection,
-               "GetSelectionForCopy succeeded, but the result is null");
+  // Flushing notifications can cause mPresShell to be destroyed (bug 577963).
+  NS_ENSURE_TRUE(!mPresShell->IsDestroying(), NS_ERROR_FAILURE);
 
-  PRBool isCollapsed;
-  rv = mSelection->GetIsCollapsed(&isCollapsed);
-  if (NS_FAILED(rv))
-    return NS_ERROR_NOT_AVAILABLE;
-  aEvent->mReply.mHasSelection = !isCollapsed;
+  nsCopySupport::GetSelectionForCopy(mPresShell->GetDocument(),
+                                     getter_AddRefs(mSelection));
 
   nsCOMPtr<nsIDOMRange> firstRange;
-  rv = mSelection->GetRangeAt(0, getter_AddRefs(firstRange));
+  nsresult rv = mSelection->GetRangeAt(0, getter_AddRefs(firstRange));
   // This shell doesn't support selection.
   if (NS_FAILED(rv))
     return NS_ERROR_NOT_AVAILABLE;
@@ -115,22 +105,58 @@ nsContentEventHandler::Init(nsQueryContentEvent* aEvent)
 
   nsINode* startNode = mFirstSelectedRange->GetStartParent();
   NS_ENSURE_TRUE(startNode, NS_ERROR_FAILURE);
+  nsINode* endNode = mFirstSelectedRange->GetEndParent();
+  NS_ENSURE_TRUE(endNode, NS_ERROR_FAILURE);
+
+  // See bug 537041 comment 5, the range could have removed node.
+  NS_ENSURE_TRUE(startNode->GetCurrentDoc() == mPresShell->GetDocument(),
+                 NS_ERROR_NOT_AVAILABLE);
+  NS_ASSERTION(startNode->GetCurrentDoc() == endNode->GetCurrentDoc(),
+               "mFirstSelectedRange crosses the document boundary");
+
   mRootContent = startNode->GetSelectionRootContent(mPresShell);
   NS_ENSURE_TRUE(mRootContent, NS_ERROR_FAILURE);
+  return NS_OK;
+}
+
+nsresult
+nsContentEventHandler::Init(nsQueryContentEvent* aEvent)
+{
+  NS_ASSERTION(aEvent, "aEvent must not be null");
+
+  nsresult rv = InitCommon();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  aEvent->mSucceeded = PR_FALSE;
 
   aEvent->mReply.mContentsRoot = mRootContent.get();
 
-  nsRefPtr<nsCaret> caret;
-  rv = mPresShell->GetCaret(getter_AddRefs(caret));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(caret, "GetCaret succeeded, but the result is null");
+  PRBool isCollapsed;
+  rv = mSelection->GetIsCollapsed(&isCollapsed);
+  NS_ENSURE_SUCCESS(rv, NS_ERROR_NOT_AVAILABLE);
+  aEvent->mReply.mHasSelection = !isCollapsed;
+
+  nsRefPtr<nsCaret> caret = mPresShell->GetCaret();
+  NS_ASSERTION(caret, "GetCaret returned null");
+
   nsRect r;
-  nsIView* view = nsnull;
-  rv = caret->GetCaretCoordinates(nsCaret::eRenderingViewCoordinates,
-                                  mSelection, &r, &isCollapsed, &view);
+  nsIFrame* frame = caret->GetGeometry(mSelection, &r);
+  NS_ENSURE_TRUE(frame, NS_ERROR_FAILURE);
+
+  aEvent->mReply.mFocusedWidget = frame->GetNearestWidget();
+
+  return NS_OK;
+}
+
+nsresult
+nsContentEventHandler::Init(nsSelectionEvent* aEvent)
+{
+  NS_ASSERTION(aEvent, "aEvent must not be null");
+
+  nsresult rv = InitCommon();
   NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(view, NS_ERROR_FAILURE);
-  aEvent->mReply.mFocusedWidget = view->GetWidget();
+
+  aEvent->mSucceeded = PR_FALSE;
 
   return NS_OK;
 }
@@ -141,7 +167,7 @@ nsContentEventHandler::Init(nsQueryContentEvent* aEvent)
 // we don't want to include the bogus BRs at the end.
 static PRBool IsContentBR(nsIContent* aContent)
 {
-  return aContent->IsNodeOfType(nsINode::eHTML) &&
+  return aContent->IsHTML() &&
          aContent->Tag() == nsGkAtoms::br &&
          !aContent->AttrValueIs(kNameSpaceID_None,
                                 nsGkAtoms::type,
@@ -494,8 +520,7 @@ static nsINode* AdjustTextRectNode(nsINode* aNode,
 
 // Similar to nsFrameSelection::GetFrameForNodeOffset,
 // but this is more flexible for OnQueryTextRect to use
-static nsresult GetFrameForTextRect(nsIPresShell* aPresShell,
-                                    nsINode* aNode,
+static nsresult GetFrameForTextRect(nsINode* aNode,
                                     PRInt32 aOffset,
                                     PRBool aHint,
                                     nsIFrame** aReturnFrame)
@@ -503,7 +528,7 @@ static nsresult GetFrameForTextRect(nsIPresShell* aPresShell,
   NS_ENSURE_TRUE(aNode && aNode->IsNodeOfType(nsINode::eCONTENT),
                  NS_ERROR_UNEXPECTED);
   nsIContent* content = static_cast<nsIContent*>(aNode);
-  nsIFrame* frame = aPresShell->GetPrimaryFrameFor(content);
+  nsIFrame* frame = content->GetPrimaryFrame();
   NS_ENSURE_TRUE(frame, NS_ERROR_FAILURE);
   PRInt32 childOffset = 0;
   return frame->GetChildFrameContainingOffset(aOffset, aHint, &childOffset,
@@ -539,8 +564,7 @@ nsContentEventHandler::OnQueryTextRect(nsQueryContentEvent* aEvent)
     node = AdjustTextRectNode(range->GetStartParent(), offset);
   }
   nsIFrame* firstFrame = nsnull;
-  rv = GetFrameForTextRect(mPresShell, node, offset,
-                           PR_TRUE, &firstFrame);
+  rv = GetFrameForTextRect(node, offset, PR_TRUE, &firstFrame);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // get the starting frame rect
@@ -558,8 +582,7 @@ nsContentEventHandler::OnQueryTextRect(nsQueryContentEvent* aEvent)
   offset = range->EndOffset();
   node = AdjustTextRectNode(range->GetEndParent(), offset);
   nsIFrame* lastFrame = nsnull;
-  rv = GetFrameForTextRect(mPresShell, node, offset,
-                           range->Collapsed(), &lastFrame);
+  rv = GetFrameForTextRect(node, offset, range->Collapsed(), &lastFrame);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // iterate over all covered frames
@@ -571,7 +594,7 @@ nsContentEventHandler::OnQueryTextRect(nsQueryContentEvent* aEvent)
         node = iter->GetCurrentNode();
         if (!node || !node->IsNodeOfType(nsINode::eCONTENT))
           continue;
-        frame = mPresShell->GetPrimaryFrameFor(static_cast<nsIContent*>(node));
+        frame = static_cast<nsIContent*>(node)->GetPrimaryFrame();
       } while (!frame && !iter->IsDone());
       if (!frame) {
         // this can happen when the end offset of the range is 0.
@@ -610,7 +633,7 @@ nsContentEventHandler::OnQueryEditorRect(nsQueryContentEvent* aEvent)
   if (NS_FAILED(rv))
     return rv;
 
-  nsIFrame* frame = mPresShell->GetPrimaryFrameFor(mRootContent);
+  nsIFrame* frame = mRootContent->GetPrimaryFrame();
   NS_ENSURE_TRUE(frame, NS_ERROR_FAILURE);
 
   // get rect for first frame
@@ -639,10 +662,8 @@ nsContentEventHandler::OnQueryCaretRect(nsQueryContentEvent* aEvent)
   if (NS_FAILED(rv))
     return rv;
 
-  nsRefPtr<nsCaret> caret;
-  rv = mPresShell->GetCaret(getter_AddRefs(caret));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(caret, "GetCaret succeeded, but the result is null");
+  nsRefPtr<nsCaret> caret = mPresShell->GetCaret();
+  NS_ASSERTION(caret, "GetCaret returned null");
 
   // When the selection is collapsed and the queried offset is current caret
   // position, we should return the "real" caret rect.
@@ -655,14 +676,14 @@ nsContentEventHandler::OnQueryCaretRect(nsQueryContentEvent* aEvent)
     rv = GetFlatTextOffsetOfRange(mRootContent, mFirstSelectedRange, &offset);
     NS_ENSURE_SUCCESS(rv, rv);
     if (offset == aEvent->mInput.mOffset) {
-      PRBool isCollapsed;
       nsRect rect;
-      rv = caret->GetCaretCoordinates(nsCaret::eTopLevelWindowCoordinates,
-                                      mSelection, &rect,
-                                      &isCollapsed, nsnull);
-      aEvent->mReply.mRect =
-          rect.ToOutsidePixels(mPresContext->AppUnitsPerDevPixel());
+      nsIFrame* caretFrame = caret->GetGeometry(mSelection, &rect);
+      if (!caretFrame)
+        return NS_ERROR_FAILURE;
+      rv = ConvertToRootViewRelativeOffset(caretFrame, rect);
       NS_ENSURE_SUCCESS(rv, rv);
+      aEvent->mReply.mRect =
+        rect.ToOutsidePixels(caretFrame->PresContext()->AppUnitsPerDevPixel());
       aEvent->mSucceeded = PR_TRUE;
       return NS_OK;
     }
@@ -741,16 +762,47 @@ nsContentEventHandler::OnQueryCharacterAtPoint(nsQueryContentEvent* aEvent)
     return rv;
 
   nsIFrame* rootFrame = mPresShell->GetRootFrame();
+  NS_ENSURE_TRUE(rootFrame, NS_ERROR_FAILURE);
+  nsIWidget* rootWidget = rootFrame->GetNearestWidget();
+  NS_ENSURE_TRUE(rootWidget, NS_ERROR_FAILURE);
+
+  // The root frame's widget might be different, e.g., the event was fired on
+  // a popup but the rootFrame is the document root.
+  if (rootWidget != aEvent->widget) {
+    NS_PRECONDITION(aEvent->widget, "The event must have the widget");
+    nsIView* view = nsIView::GetViewFor(aEvent->widget);
+    NS_ENSURE_TRUE(view, NS_ERROR_FAILURE);
+    rootFrame = nsLayoutUtils::GetFrameFor(view);
+    NS_ENSURE_TRUE(rootFrame, NS_ERROR_FAILURE);
+    rootWidget = rootFrame->GetNearestWidget();
+    NS_ENSURE_TRUE(rootWidget, NS_ERROR_FAILURE);
+  }
+
+  nsQueryContentEvent eventOnRoot(PR_TRUE, NS_QUERY_CHARACTER_AT_POINT,
+                                  rootWidget);
+  eventOnRoot.refPoint = aEvent->refPoint;
+  if (rootWidget != aEvent->widget) {
+    eventOnRoot.refPoint += aEvent->widget->WidgetToScreenOffset();
+    eventOnRoot.refPoint -= rootWidget->WidgetToScreenOffset();
+  }
   nsPoint ptInRoot =
-    nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent, rootFrame);
+    nsLayoutUtils::GetEventCoordinatesRelativeTo(&eventOnRoot, rootFrame);
+
   nsIFrame* targetFrame = nsLayoutUtils::GetFrameForPoint(rootFrame, ptInRoot);
-  if (!targetFrame || targetFrame->GetType() != nsGkAtoms::textFrame) {
+  if (!targetFrame || targetFrame->GetType() != nsGkAtoms::textFrame ||
+      !targetFrame->GetContent() ||
+      !nsContentUtils::ContentIsDescendantOf(targetFrame->GetContent(),
+                                             mRootContent)) {
     // there is no character at the point.
     aEvent->mReply.mOffset = nsQueryContentEvent::NOT_FOUND;
     aEvent->mSucceeded = PR_TRUE;
     return NS_OK;
   }
-  nsPoint ptInTarget = ptInRoot - targetFrame->GetOffsetTo(rootFrame);
+  nsPoint ptInTarget = ptInRoot + rootFrame->GetOffsetToCrossDoc(targetFrame);
+  PRInt32 rootAPD = rootFrame->PresContext()->AppUnitsPerDevPixel();
+  PRInt32 targetAPD = targetFrame->PresContext()->AppUnitsPerDevPixel();
+  ptInTarget = ptInTarget.ConvertAppUnits(rootAPD, targetAPD);
+
   nsTextFrame* textframe = static_cast<nsTextFrame*>(targetFrame);
   nsIFrame::ContentOffsets offsets =
     textframe->GetCharacterOffsetAtFramePoint(ptInTarget);
@@ -773,6 +825,52 @@ nsContentEventHandler::OnQueryCharacterAtPoint(nsQueryContentEvent* aEvent)
   return NS_OK;
 }
 
+nsresult
+nsContentEventHandler::OnQueryDOMWidgetHittest(nsQueryContentEvent* aEvent)
+{
+  nsresult rv = Init(aEvent);
+  if (NS_FAILED(rv))
+    return rv;
+
+  aEvent->mReply.mWidgetIsHit = PR_FALSE;
+
+  NS_ENSURE_TRUE(aEvent->widget, NS_ERROR_FAILURE);
+
+  nsIDocument* doc = mPresShell->GetDocument();
+  NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
+  nsIFrame* docFrame = mPresShell->GetRootFrame();
+  NS_ENSURE_TRUE(docFrame, NS_ERROR_FAILURE);
+
+  nsIntPoint eventLoc =
+    aEvent->refPoint + aEvent->widget->WidgetToScreenOffset();
+  nsIntRect docFrameRect = docFrame->GetScreenRect(); // Returns CSS pixels
+  eventLoc.x = mPresContext->DevPixelsToIntCSSPixels(eventLoc.x);
+  eventLoc.y = mPresContext->DevPixelsToIntCSSPixels(eventLoc.y);
+  eventLoc.x -= docFrameRect.x;
+  eventLoc.y -= docFrameRect.y;
+
+  nsCOMPtr<nsIDOMElement> elementUnderMouse;
+  doc->ElementFromPointHelper(eventLoc.x, eventLoc.y, PR_FALSE, PR_FALSE,
+                              getter_AddRefs(elementUnderMouse));
+  nsCOMPtr<nsIContent> contentUnderMouse = do_QueryInterface(elementUnderMouse);
+  if (contentUnderMouse) {
+    nsIWidget* targetWidget = nsnull;
+    nsIFrame* targetFrame = contentUnderMouse->GetPrimaryFrame();
+    nsIObjectFrame* pluginFrame = do_QueryFrame(targetFrame);
+    if (pluginFrame) {
+      targetWidget = pluginFrame->GetWidget();
+    } else if (targetFrame) {
+      targetWidget = targetFrame->GetNearestWidget();
+    }
+    if (aEvent->widget == targetWidget)
+      aEvent->mReply.mWidgetIsHit = PR_TRUE;
+    nsIWidget* pEventWidget = aEvent->widget.get();
+  }
+
+  aEvent->mSucceeded = PR_TRUE;
+  return NS_OK;
+}
+
 nsresult
 nsContentEventHandler::GetFlatTextOffsetOfRange(nsIContent* aRootContent,
                                                 nsINode* aNode,
@@ -859,15 +957,15 @@ static void AdjustRangeForSelection(nsIContent* aRoot,
     node = node->GetParent();
     offset = node->IndexOf(*aNode) + (offset ? 1 : 0);
   }
-  nsINode* brNode = node->GetChildAt(offset - 1);
-  while (brNode && brNode->IsNodeOfType(nsINode::eHTML)) {
-    nsIContent* brContent = static_cast<nsIContent*>(brNode);
+  
+  nsIContent* brContent = node->GetChildAt(offset - 1);
+  while (brContent && brContent->IsHTML()) {
     if (brContent->Tag() != nsGkAtoms::br || IsContentBR(brContent))
       break;
-    brNode = node->GetChildAt(--offset - 1);
+    brContent = node->GetChildAt(--offset - 1);
   }
   *aNode = node;
-  *aOffset = PR_MAX(offset, 0);
+  *aOffset = NS_MAX(offset, 0);
 }
 
 nsresult
@@ -876,17 +974,23 @@ nsContentEventHandler::OnSelectionEvent(nsSelectionEvent* aEvent)
   aEvent->mSucceeded = PR_FALSE;
 
   // Get selection to manipulate
-  nsCOMPtr<nsISelection> sel;
+  // XXX why do we need to get them from ISM? This method should work fine
+  //     without ISM.
   nsresult rv = nsIMEStateManager::
-      GetFocusSelectionAndRoot(getter_AddRefs(sel),
+      GetFocusSelectionAndRoot(getter_AddRefs(mSelection),
                                getter_AddRefs(mRootContent));
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (rv != NS_ERROR_NOT_AVAILABLE) {
+    NS_ENSURE_SUCCESS(rv, rv);
+  } else {
+    rv = Init(aEvent);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   // Get range from offset and length
   nsRefPtr<nsRange> range = new nsRange();
   NS_ENSURE_TRUE(range, NS_ERROR_OUT_OF_MEMORY);
-  rv = SetRangeFromFlatTextOffset(range, aEvent->mOffset,
-                                  aEvent->mLength, PR_TRUE);
+  rv = SetRangeFromFlatTextOffset(range, aEvent->mOffset, aEvent->mLength,
+                                  aEvent->mExpandToClusterBoundary);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsINode* startNode = range->GetStartParent();
@@ -900,32 +1004,32 @@ nsContentEventHandler::OnSelectionEvent(nsSelectionEvent* aEvent)
   nsCOMPtr<nsIDOMNode> endDomNode(do_QueryInterface(endNode));
   NS_ENSURE_TRUE(startDomNode && endDomNode, NS_ERROR_UNEXPECTED);
 
-  nsCOMPtr<nsISelectionPrivate> selPrivate = do_QueryInterface(sel);
+  nsCOMPtr<nsISelectionPrivate> selPrivate = do_QueryInterface(mSelection);
   NS_ENSURE_TRUE(selPrivate, NS_ERROR_UNEXPECTED);
   selPrivate->StartBatchChanges();
 
   // Clear selection first before setting
-  rv = sel->RemoveAllRanges();
+  rv = mSelection->RemoveAllRanges();
   // Need to call EndBatchChanges at the end even if call failed
   if (NS_SUCCEEDED(rv)) {
     if (aEvent->mReversed) {
-      rv = sel->Collapse(endDomNode, endOffset);
+      rv = mSelection->Collapse(endDomNode, endOffset);
     } else {
-      rv = sel->Collapse(startDomNode, startOffset);
+      rv = mSelection->Collapse(startDomNode, startOffset);
     }
     if (NS_SUCCEEDED(rv) &&
         (startDomNode != endDomNode || startOffset != endOffset)) {
       if (aEvent->mReversed) {
-        rv = sel->Extend(startDomNode, startOffset);
+        rv = mSelection->Extend(startDomNode, startOffset);
       } else {
-        rv = sel->Extend(endDomNode, endOffset);
+        rv = mSelection->Extend(endDomNode, endOffset);
       }
     }
   }
   selPrivate->EndBatchChanges();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISelection2>(do_QueryInterface(sel))->ScrollIntoView(
+  nsCOMPtr<nsISelection2>(do_QueryInterface(mSelection))->ScrollIntoView(
       nsISelectionController::SELECTION_FOCUS_REGION, PR_FALSE, -1, -1);
   aEvent->mSucceeded = PR_TRUE;
   return NS_OK;
