@@ -44,15 +44,16 @@
 #include "jsstdint.h"
 #include "jsdtoa.h"
 #include "jsprf.h"
-#include "jsutil.h" /* Added by JSIFY */
+#include "jsapi.h"
 #include "jsprvtd.h"
 #include "jsnum.h"
 #include "jsbit.h"
 #include "jslibmath.h"
+#include "jscntxt.h"
 
-#ifdef JS_THREADSAFE
-#include "jslock.h"
-#endif
+#include "jsobjinlines.h"
+
+using namespace js;
 
 #ifdef IS_LITTLE_ENDIAN
 #define IEEE_8087
@@ -78,46 +79,18 @@
 #endif
 */
 
-#ifdef JS_THREADSAFE
-static PRLock *dtoalock;
-static JSBool _dtoainited = JS_FALSE;
+/*
+ * MALLOC gets declared external, and that doesn't work for class members, so
+ * wrap.
+ */
+inline void* dtoa_malloc(size_t size) { return OffTheBooks::malloc_(size); }
+inline void dtoa_free(void* p) { return UnwantedForeground::free_(p); }
 
-#define LOCK_DTOA() PR_Lock(dtoalock);
-#define UNLOCK_DTOA() PR_Unlock(dtoalock)
-#else
-#define LOCK_DTOA()
-#define UNLOCK_DTOA()
-#endif
+#define NO_GLOBAL_STATE
+#define MALLOC dtoa_malloc
+#define FREE dtoa_free
 #include "dtoa.c"
 
-JS_FRIEND_API(JSBool)
-js_InitDtoa()
-{
-#ifdef JS_THREADSAFE
-    if (!_dtoainited) {
-        dtoalock = PR_NewLock();
-        JS_ASSERT(dtoalock);
-        _dtoainited = JS_TRUE;
-    }
-
-    return (dtoalock != 0);
-#else
-    return JS_TRUE;
-#endif
-}
-
-JS_FRIEND_API(void)
-js_FinishDtoa()
-{
-#ifdef JS_THREADSAFE
-    if (_dtoainited) {
-        PR_DestroyLock(dtoalock);
-        dtoalock = NULL;
-        _dtoainited = JS_FALSE;
-    }
-#endif
-}
-
 /* Mapping of JSDToStrMode -> js_dtoa mode */
 static const uint8 dtoaModes[] = {
     0,   /* DTOSTR_STANDARD */
@@ -126,20 +99,19 @@ static const uint8 dtoaModes[] = {
     2,   /* DTOSTR_EXPONENTIAL, */
     2};  /* DTOSTR_PRECISION */
 
-JS_FRIEND_API(double)
-JS_strtod(const char *s00, char **se, int *err)
+double
+js_strtod_harder(DtoaState *state, const char *s00, char **se, int *err)
 {
     double retval;
     if (err)
         *err = 0;
-    LOCK_DTOA();
-    retval = _strtod(s00, se);
-    UNLOCK_DTOA();
+    retval = _strtod(state, s00, se);
     return retval;
 }
 
-JS_FRIEND_API(char *)
-JS_dtostr(char *buffer, size_t bufferSize, JSDToStrMode mode, int precision, double dinput)
+char *
+js_dtostr(DtoaState *state, char *buffer, size_t bufferSize, JSDToStrMode mode, int precision,
+          double dinput)
 {
     U d;
     int decPt;        /* Offset of decimal point from first digit */
@@ -159,24 +131,20 @@ JS_dtostr(char *buffer, size_t bufferSize, JSDToStrMode mode, int precision, dou
     if (mode == DTOSTR_FIXED && (dinput >= 1e21 || dinput <= -1e21))
         mode = DTOSTR_STANDARD;
 
-    LOCK_DTOA();
     dval(d) = dinput;
-    numBegin = dtoa(d, dtoaModes[mode], precision, &decPt, &sign, &numEnd);
+    numBegin = dtoa(PASS_STATE d, dtoaModes[mode], precision, &decPt, &sign, &numEnd);
     if (!numBegin) {
-        UNLOCK_DTOA();
         return NULL;
     }
 
     nDigits = numEnd - numBegin;
     JS_ASSERT((size_t) nDigits <= bufferSize - 2);
     if ((size_t) nDigits > bufferSize - 2) {
-        UNLOCK_DTOA();
         return NULL;
     }
 
     memcpy(buffer + 2, numBegin, nDigits);
-    freedtoa(numBegin);
-    UNLOCK_DTOA();
+    freedtoa(PASS_STATE numBegin);
     numBegin = buffer + 2; /* +2 leaves space for sign and/or decimal point */
     numEnd = numBegin + nDigits;
     *numEnd = '\0';
@@ -353,8 +321,8 @@ static uint32 quorem2(Bigint *b, int32 k)
 #define DTOBASESTR_BUFFER_SIZE 1078
 #define BASEDIGIT(digit) ((char)(((digit) >= 10) ? 'a' - 10 + (digit) : '0' + (digit)))
 
-JS_FRIEND_API(char *)
-JS_dtobasestr(int base, double dinput)
+char *
+js_dtobasestr(DtoaState *state, int base, double dinput)
 {
     U d;
     char *buffer;        /* The output string */
@@ -368,205 +336,214 @@ JS_dtobasestr(int base, double dinput)
     JS_ASSERT(base >= 2 && base <= 36);
 
     dval(d) = dinput;
-    buffer = (char*) js_malloc(DTOBASESTR_BUFFER_SIZE);
-    if (buffer) {
-        p = buffer;
-        if (dval(d) < 0.0
+    buffer = (char*) OffTheBooks::malloc_(DTOBASESTR_BUFFER_SIZE);
+    if (!buffer)
+        return NULL;
+    p = buffer;
+
+    if (dval(d) < 0.0
 #if defined(XP_WIN) || defined(XP_OS2)
-            && !((word0(d) & Exp_mask) == Exp_mask && ((word0(d) & Frac_mask) || word1(d))) /* Visual C++ doesn't know how to compare against NaN */
+        && !((word0(d) & Exp_mask) == Exp_mask && ((word0(d) & Frac_mask) || word1(d))) /* Visual C++ doesn't know how to compare against NaN */
 #endif
-           ) {
-            *p++ = '-';
-            dval(d) = -dval(d);
-        }
+       ) {
+        *p++ = '-';
+        dval(d) = -dval(d);
+    }
 
-        /* Check for Infinity and NaN */
-        if ((word0(d) & Exp_mask) == Exp_mask) {
-            strcpy(p, !word1(d) && !(word0(d) & Frac_mask) ? "Infinity" : "NaN");
-            return buffer;
-        }
+    /* Check for Infinity and NaN */
+    if ((word0(d) & Exp_mask) == Exp_mask) {
+        strcpy(p, !word1(d) && !(word0(d) & Frac_mask) ? "Infinity" : "NaN");
+        return buffer;
+    }
 
-        LOCK_DTOA();
-        /* Output the integer part of d with the digits in reverse order. */
-        pInt = p;
-        dval(di) = floor(dval(d));
-        if (dval(di) <= 4294967295.0) {
-            uint32 n = (uint32)dval(di);
-            if (n)
-                do {
-                    uint32 m = n / base;
-                    digit = n - m*base;
-                    n = m;
-                    JS_ASSERT(digit < (uint32)base);
-                    *p++ = BASEDIGIT(digit);
-                } while (n);
-            else *p++ = '0';
-        } else {
-            int e;
-            int bits;  /* Number of significant bits in di; not used. */
-            Bigint *b = d2b(di, &e, &bits);
-            if (!b)
-                goto nomem1;
-            b = lshift(b, e);
-            if (!b) {
-              nomem1:
-                Bfree(b);
-                UNLOCK_DTOA();
-                js_free(buffer);
-                return NULL;
-            }
+    /* Output the integer part of d with the digits in reverse order. */
+    pInt = p;
+    dval(di) = floor(dval(d));
+    if (dval(di) <= 4294967295.0) {
+        uint32 n = (uint32)dval(di);
+        if (n)
             do {
-                digit = divrem(b, base);
+                uint32 m = n / base;
+                digit = n - m*base;
+                n = m;
                 JS_ASSERT(digit < (uint32)base);
                 *p++ = BASEDIGIT(digit);
-            } while (b->wds);
-            Bfree(b);
+            } while (n);
+        else *p++ = '0';
+    } else {
+        int e;
+        int bits;  /* Number of significant bits in di; not used. */
+        Bigint *b = d2b(PASS_STATE di, &e, &bits);
+        if (!b)
+            goto nomem1;
+        b = lshift(PASS_STATE b, e);
+        if (!b) {
+          nomem1:
+            Bfree(PASS_STATE b);
+            UnwantedForeground::free_(buffer);
+            return NULL;
         }
-        /* Reverse the digits of the integer part of d. */
-        q = p-1;
-        while (q > pInt) {
-            char ch = *pInt;
-            *pInt++ = *q;
-            *q-- = ch;
+        do {
+            digit = divrem(b, base);
+            JS_ASSERT(digit < (uint32)base);
+            *p++ = BASEDIGIT(digit);
+        } while (b->wds);
+        Bfree(PASS_STATE b);
+    }
+    /* Reverse the digits of the integer part of d. */
+    q = p-1;
+    while (q > pInt) {
+        char ch = *pInt;
+        *pInt++ = *q;
+        *q-- = ch;
+    }
+
+    dval(df) = dval(d) - dval(di);
+    if (dval(df) != 0.0) {
+        /* We have a fraction. */
+        int e, bbits;
+        int32 s2, done;
+        Bigint *b, *s, *mlo, *mhi;
+
+        b = s = mlo = mhi = NULL;
+
+        *p++ = '.';
+        b = d2b(PASS_STATE df, &e, &bbits);
+        if (!b) {
+          nomem2:
+            Bfree(PASS_STATE b);
+            Bfree(PASS_STATE s);
+            if (mlo != mhi)
+                Bfree(PASS_STATE mlo);
+            Bfree(PASS_STATE mhi);
+            UnwantedForeground::free_(buffer);
+            return NULL;
         }
+        JS_ASSERT(e < 0);
+        /* At this point df = b * 2^e.  e must be less than zero because 0 < df < 1. */
 
-        dval(df) = dval(d) - dval(di);
-        if (dval(df) != 0.0) {
-            /* We have a fraction. */
-            int e, bbits;
-            int32 s2, done;
-            Bigint *b, *s, *mlo, *mhi;
-
-            b = s = mlo = mhi = NULL;
-
-            *p++ = '.';
-            b = d2b(df, &e, &bbits);
-            if (!b) {
-              nomem2:
-                Bfree(b);
-                Bfree(s);
-                if (mlo != mhi)
-                    Bfree(mlo);
-                Bfree(mhi);
-                UNLOCK_DTOA();
-                js_free(buffer);
-                return NULL;
-            }
-            JS_ASSERT(e < 0);
-            /* At this point df = b * 2^e.  e must be less than zero because 0 < df < 1. */
-
-            s2 = -(int32)(word0(d) >> Exp_shift1 & Exp_mask>>Exp_shift1);
+        s2 = -(int32)(word0(d) >> Exp_shift1 & Exp_mask>>Exp_shift1);
 #ifndef Sudden_Underflow
-            if (!s2)
-                s2 = -1;
+        if (!s2)
+            s2 = -1;
 #endif
-            s2 += Bias + P;
-            /* 1/2^s2 = (nextDouble(d) - d)/2 */
-            JS_ASSERT(-s2 < e);
-            mlo = i2b(1);
-            if (!mlo)
+        s2 += Bias + P;
+        /* 1/2^s2 = (nextDouble(d) - d)/2 */
+        JS_ASSERT(-s2 < e);
+        mlo = i2b(PASS_STATE 1);
+        if (!mlo)
+            goto nomem2;
+        mhi = mlo;
+        if (!word1(d) && !(word0(d) & Bndry_mask)
+#ifndef Sudden_Underflow
+            && word0(d) & (Exp_mask & Exp_mask << 1)
+#endif
+            ) {
+            /* The special case.  Here we want to be within a quarter of the last input
+               significant digit instead of one half of it when the output string's value is less than d.  */
+            s2 += Log2P;
+            mhi = i2b(PASS_STATE 1<<Log2P);
+            if (!mhi)
                 goto nomem2;
-            mhi = mlo;
-            if (!word1(d) && !(word0(d) & Bndry_mask)
-#ifndef Sudden_Underflow
-                && word0(d) & (Exp_mask & Exp_mask << 1)
-#endif
-                ) {
-                /* The special case.  Here we want to be within a quarter of the last input
-                   significant digit instead of one half of it when the output string's value is less than d.  */
-                s2 += Log2P;
-                mhi = i2b(1<<Log2P);
+        }
+        b = lshift(PASS_STATE b, e + s2);
+        if (!b)
+            goto nomem2;
+        s = i2b(PASS_STATE 1);
+        if (!s)
+            goto nomem2;
+        s = lshift(PASS_STATE s, s2);
+        if (!s)
+            goto nomem2;
+        /* At this point we have the following:
+         *   s = 2^s2;
+         *   1 > df = b/2^s2 > 0;
+         *   (d - prevDouble(d))/2 = mlo/2^s2;
+         *   (nextDouble(d) - d)/2 = mhi/2^s2. */
+
+        done = JS_FALSE;
+        do {
+            int32 j, j1;
+            Bigint *delta;
+
+            b = multadd(PASS_STATE b, base, 0);
+            if (!b)
+                goto nomem2;
+            digit = quorem2(b, s2);
+            if (mlo == mhi) {
+                mlo = mhi = multadd(PASS_STATE mlo, base, 0);
                 if (!mhi)
                     goto nomem2;
             }
-            b = lshift(b, e + s2);
-            if (!b)
-                goto nomem2;
-            s = i2b(1);
-            if (!s)
-                goto nomem2;
-            s = lshift(s, s2);
-            if (!s)
-                goto nomem2;
-            /* At this point we have the following:
-             *   s = 2^s2;
-             *   1 > df = b/2^s2 > 0;
-             *   (d - prevDouble(d))/2 = mlo/2^s2;
-             *   (nextDouble(d) - d)/2 = mhi/2^s2. */
-
-            done = JS_FALSE;
-            do {
-                int32 j, j1;
-                Bigint *delta;
-
-                b = multadd(b, base, 0);
-                if (!b)
+            else {
+                mlo = multadd(PASS_STATE mlo, base, 0);
+                if (!mlo)
                     goto nomem2;
-                digit = quorem2(b, s2);
-                if (mlo == mhi) {
-                    mlo = mhi = multadd(mlo, base, 0);
-                    if (!mhi)
-                        goto nomem2;
-                }
-                else {
-                    mlo = multadd(mlo, base, 0);
-                    if (!mlo)
-                        goto nomem2;
-                    mhi = multadd(mhi, base, 0);
-                    if (!mhi)
-                        goto nomem2;
-                }
-
-                /* Do we yet have the shortest string that will round to d? */
-                j = cmp(b, mlo);
-                /* j is b/2^s2 compared with mlo/2^s2. */
-                delta = diff(s, mhi);
-                if (!delta)
+                mhi = multadd(PASS_STATE mhi, base, 0);
+                if (!mhi)
                     goto nomem2;
-                j1 = delta->sign ? 1 : cmp(b, delta);
-                Bfree(delta);
-                /* j1 is b/2^s2 compared with 1 - mhi/2^s2. */
+            }
+
+            /* Do we yet have the shortest string that will round to d? */
+            j = cmp(b, mlo);
+            /* j is b/2^s2 compared with mlo/2^s2. */
+            delta = diff(PASS_STATE s, mhi);
+            if (!delta)
+                goto nomem2;
+            j1 = delta->sign ? 1 : cmp(b, delta);
+            Bfree(PASS_STATE delta);
+            /* j1 is b/2^s2 compared with 1 - mhi/2^s2. */
 
 #ifndef ROUND_BIASED
-                if (j1 == 0 && !(word1(d) & 1)) {
-                    if (j > 0)
-                        digit++;
-                    done = JS_TRUE;
-                } else
-#endif
-                if (j < 0 || (j == 0
-#ifndef ROUND_BIASED
-                    && !(word1(d) & 1)
-#endif
-                    )) {
-                    if (j1 > 0) {
-                        /* Either dig or dig+1 would work here as the least significant digit.
-                           Use whichever would produce an output value closer to d. */
-                        b = lshift(b, 1);
-                        if (!b)
-                            goto nomem2;
-                        j1 = cmp(b, s);
-                        if (j1 > 0) /* The even test (|| (j1 == 0 && (digit & 1))) is not here because it messes up odd base output
-                                     * such as 3.5 in base 3.  */
-                            digit++;
-                    }
-                    done = JS_TRUE;
-                } else if (j1 > 0) {
+            if (j1 == 0 && !(word1(d) & 1)) {
+                if (j > 0)
                     digit++;
-                    done = JS_TRUE;
+                done = JS_TRUE;
+            } else
+#endif
+            if (j < 0 || (j == 0
+#ifndef ROUND_BIASED
+                && !(word1(d) & 1)
+#endif
+                )) {
+                if (j1 > 0) {
+                    /* Either dig or dig+1 would work here as the least significant digit.
+                       Use whichever would produce an output value closer to d. */
+                    b = lshift(PASS_STATE b, 1);
+                    if (!b)
+                        goto nomem2;
+                    j1 = cmp(b, s);
+                    if (j1 > 0) /* The even test (|| (j1 == 0 && (digit & 1))) is not here because it messes up odd base output
+                                 * such as 3.5 in base 3.  */
+                        digit++;
                 }
-                JS_ASSERT(digit < (uint32)base);
-                *p++ = BASEDIGIT(digit);
-            } while (!done);
-            Bfree(b);
-            Bfree(s);
-            if (mlo != mhi)
-                Bfree(mlo);
-            Bfree(mhi);
-        }
-        JS_ASSERT(p < buffer + DTOBASESTR_BUFFER_SIZE);
-        *p = '\0';
-        UNLOCK_DTOA();
+                done = JS_TRUE;
+            } else if (j1 > 0) {
+                digit++;
+                done = JS_TRUE;
+            }
+            JS_ASSERT(digit < (uint32)base);
+            *p++ = BASEDIGIT(digit);
+        } while (!done);
+        Bfree(PASS_STATE b);
+        Bfree(PASS_STATE s);
+        if (mlo != mhi)
+            Bfree(PASS_STATE mlo);
+        Bfree(PASS_STATE mhi);
     }
+    JS_ASSERT(p < buffer + DTOBASESTR_BUFFER_SIZE);
+    *p = '\0';
     return buffer;
 }
+
+DtoaState *
+js_NewDtoaState()
+{
+    return newdtoa();
+}
+
+void
+js_DestroyDtoaState(DtoaState *state)
+{
+    destroydtoa(state);
+}
