@@ -41,50 +41,26 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("13.4.4.35 - setName");
+START("13.4.4.37 - XML text()");
 
-TEST(1, true, XML.prototype.hasOwnProperty("setName"));
+TEST(1, true, XML.prototype.hasOwnProperty("text"));
 
 x =
 <alpha>
     <bravo>one</bravo>
+    <charlie>
+        <bravo>two</bravo>
+    </charlie>
 </alpha>;
 
-correct =
-<charlie>
-    <bravo>one</bravo>
-</charlie>;
+TEST_XML(2, "one", x.bravo.text());
 
-x.setName("charlie");
-
-TEST(2, correct, x);
-
-x =
-<ns:alpha xmlns:ns="http://foobar/">
-    <ns:bravo>one</ns:bravo>
-</ns:alpha>;
-
-correct =
-<charlie xmlns:ns="http://foobar/">
-    <ns:bravo>one</ns:bravo>
-</charlie>;
-
-x.setName("charlie");
-
-TEST(3, correct, x);
-
-x =
-<ns:alpha xmlns:ns="http://foobar/">
-    <ns:bravo>one</ns:bravo>
-</ns:alpha>;
-
-correct =
-<ns:charlie xmlns:ns="http://foobar/">
-    <ns:bravo>one</ns:bravo>
-</ns:charlie>;
-
-x.setName(new QName("http://foobar/", "charlie"));
-
-TEST(4, correct, x);
+correct = new XMLList();
+correct += new XML("one");
+correct += new XML("two");
+TEST(3, correct, x..bravo.text());   
+TEST_XML(4, "", x.charlie.text());
+TEST_XML(5, "", x.foobar.text());
+TEST_XML(6, "one", x.*.text());
 
 END();
