@@ -38,7 +38,7 @@
 
 
 /**
-   File Name:          7.4.2-9-n.js
+   File Name:          7.4.2-5-n.js
    ECMA Section:       7.4.2
 
    Description:
@@ -61,16 +61,16 @@
    Date:               12 november 1997
 
 */
-var SECTION = "7.4.1-9-n";
+var SECTION = "7.4.2-5-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Keywords";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var delete = true";
+DESCRIPTION = "var continue = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var delete = true",     "error",    eval("var delete = true") );
+new TestCase( SECTION,  "var continue = true",     "error",    eval("var continue = true") );
 
 test();
