@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,10 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 472450;
-var summary = 'TM: Do not assert: StackBase(fp) + blockDepth == regs.sp';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 354945;
+var summary = 'Do not crash with new Iterator';
+var expect = 'TypeError: trap __iterator__ for obj returned a primitive value';
+var actual;
 
 
 //-----------------------------------------------------------------------------
@@ -51,14 +51,14 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  jit(true);
 
-  ({__proto__: #1=[#1#]})
-    for (var y = 0; y < 3; ++y) { for each (let z in ['', function(){}]) { let x =
-                                               1, c = []; } }
-
-  jit(false);
+  try {
+    var obj = {};
+    obj.__iterator__ = function(){ };
+    for(t in (new Iterator(obj))) { }
+  } catch (ex) {
+    actual = ex.toString();
+  }
 
   reportCompare(expect, actual, summary);
 
