@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 467495;
+var summary = 'Do not crash @ js_Interpret';
 var actual = '';
 var expect = '';
 
@@ -52,28 +52,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #131 From Gary Kwong [:nth10sd]
-
-// Does not require -j:
-// =====
   try
   {
-    eval('((__defineGetter__, function (x) { function x(){} }) for');
+    eval("(function(){const y = 1; function x() '' let functional, x})();");
   }
   catch(ex)
   {
   }
-// Assertion failure: pn->pn_cookie == FREE_UPVAR_COOKIE, at ../jsparse.cpp:5698
-// =====
-  try
-  {
-    eval('( ""  ? 1.3 : x); *::*; x::x;');
-  }
-  catch(ex)
-  {
-  }
-// Assertion failure: pn != dn->dn_uses, at ../jsparse.cpp:1171
-// =====
 
   reportCompare(expect, actual, summary);
 
