@@ -43,18 +43,15 @@
 #include "nsString.h"
 #include "nsPresContext.h"
 #include "nsIStyleRule.h"
-#include "nsCRT.h"
 
 #include "nsCOMPtr.h"
 #include "nsStyleSet.h"
 #include "nsIPresShell.h"
-#include "prenv.h"
 
 #include "nsRuleNode.h"
 #include "nsStyleContext.h"
-#include "imgIRequest.h"
-
-#include "nsPrintfCString.h"
+#include "prlog.h"
+#include "nsStyleAnimation.h"
 
 #ifdef DEBUG
 // #define NOISY_DEBUG
@@ -65,6 +62,7 @@
 
 nsStyleContext::nsStyleContext(nsStyleContext* aParent,
                                nsIAtom* aPseudoTag,
+                               nsCSSPseudoElements::Type aPseudoType,
                                nsRuleNode* aRuleNode,
                                nsPresContext* aPresContext)
   : mParent(aParent),
@@ -72,9 +70,14 @@ nsStyleContext::nsStyleContext(nsStyleContext* aParent,
     mEmptyChild(nsnull),
     mPseudoTag(aPseudoTag),
     mRuleNode(aRuleNode),
-    mBits(0),
+    mAllocations(nsnull),
+    mCachedResetData(nsnull),
+    mBits(((PRUint32)aPseudoType) << NS_STYLE_CONTEXT_TYPE_SHIFT),
     mRefCnt(0)
 {
+  PR_STATIC_ASSERT((PR_UINT32_MAX >> NS_STYLE_CONTEXT_TYPE_SHIFT) >
+                   nsCSSPseudoElements::ePseudo_MAX);
+
   mNextSibling = this;
   mPrevSibling = this;
   if (mParent) {
@@ -96,6 +99,8 @@ nsStyleContext::nsStyleContext(nsStyleContext* aParent,
   NS_ASSERTION(NS_STYLE_INHERIT_MASK & NS_STYLE_INHERIT_BIT(LastItem),
                "NS_STYLE_INHERIT_MASK must be bigger, and other bits shifted");
   #undef eStyleStruct_LastItem
+
+  mRuleNode->AddRef();
 }
 
 nsStyleContext::~nsStyleContext()
@@ -104,6 +109,8 @@ nsStyleContext::~nsStyleContext()
 
   nsPresContext *presContext = mRuleNode->GetPresContext();
 
+  mRuleNode->Release();
+
   presContext->PresShell()->StyleSet()->
     NotifyStyleContextDestroyed(presContext, this);
 
@@ -113,9 +120,12 @@ nsStyleContext::~nsStyleContext()
   }
 
   // Free up our data structs.
-  if (mCachedStyleData.mResetData || mCachedStyleData.mInheritedData) {
-    mCachedStyleData.Destroy(mBits, presContext);
+  mCachedInheritedData.DestroyStructs(mBits, presContext);
+  if (mCachedResetData) {
+    mCachedResetData->Destroy(mBits, presContext);
   }
+
+  FreeAllocations(presContext);
 }
 
 void nsStyleContext::AddChild(nsStyleContext* aChild)
@@ -161,8 +171,12 @@ void nsStyleContext::RemoveChild(nsStyleContext* aChild)
 
 already_AddRefed<nsStyleContext>
 nsStyleContext::FindChildWithRules(const nsIAtom* aPseudoTag, 
-                                   nsRuleNode* aRuleNode)
+                                   nsRuleNode* aRuleNode,
+                                   nsRuleNode* aRulesIfVisited,
+                                   PRBool aRelevantLinkVisited)
 {
+  NS_ABORT_IF_FALSE(aRulesIfVisited || !aRelevantLinkVisited,
+    "aRelevantLinkVisited should only be set when we have a separate style");
   PRUint32 threshold = 10; // The # of siblings we're willing to examine
                            // before just giving this whole thing up.
 
@@ -172,9 +186,21 @@ nsStyleContext::FindChildWithRules(const nsIAtom* aPseudoTag,
   if (list) {
     nsStyleContext *child = list;
     do {
-      if (child->mRuleNode == aRuleNode && child->mPseudoTag == aPseudoTag) {
-        result = child;
-        break;
+      if (child->mRuleNode == aRuleNode &&
+          child->mPseudoTag == aPseudoTag &&
+          !child->IsStyleIfVisited() &&
+          child->RelevantLinkVisited() == aRelevantLinkVisited) {
+        PRBool match = PR_FALSE;
+        if (aRulesIfVisited) {
+          match = child->GetStyleIfVisited() &&
+                  child->GetStyleIfVisited()->mRuleNode == aRulesIfVisited;
+        } else {
+          match = !child->GetStyleIfVisited();
+        }
+        if (match) {
+          result = child;
+          break;
+        }
       }
       child = child->mNextSibling;
       threshold--;
@@ -197,60 +223,29 @@ nsStyleContext::FindChildWithRules(const nsIAtom* aPseudoTag,
   return result;
 }
 
-
-PRBool nsStyleContext::Equals(const nsStyleContext* aOther) const
+const void* nsStyleContext::GetCachedStyleData(nsStyleStructID aSID)
 {
-  PRBool  result = PR_TRUE;
-  const nsStyleContext* other = (nsStyleContext*)aOther;
-
-  if (other != this) {
-    if (mParent != other->mParent) {
-      result = PR_FALSE;
-    }
-    else if (mBits != other->mBits) {
-      result = PR_FALSE;
-    }
-    else if (mPseudoTag != other->mPseudoTag) {
-      result = PR_FALSE;
-    }
-    else if (mRuleNode != other->mRuleNode) {
-      result = PR_FALSE;
+  const void* cachedData;
+  if (nsCachedStyleData::IsReset(aSID)) {
+    if (mCachedResetData) {
+      cachedData = mCachedResetData->mStyleStructs[aSID];
+    } else {
+      cachedData = nsnull;
     }
+  } else {
+    cachedData = mCachedInheritedData.mStyleStructs[aSID];
   }
-  return result;
+  return cachedData;
 }
 
-//=========================================================================================================
-
 const void* nsStyleContext::GetStyleData(nsStyleStructID aSID)
 {
-  const void* cachedData = mCachedStyleData.GetStyleData(aSID); 
+  const void* cachedData = GetCachedStyleData(aSID);
   if (cachedData)
     return cachedData; // We have computed data stored on this node in the context tree.
   return mRuleNode->GetStyleData(aSID, this, PR_TRUE); // Our rule node will take care of it for us.
 }
 
-#define STYLE_STRUCT(name_, checkdata_cb_, ctor_args_)                      \
-  const nsStyle##name_ * nsStyleContext::GetStyle##name_ ()                 \
-  {                                                                         \
-    const nsStyle##name_ * cachedData = mCachedStyleData.GetStyle##name_(); \
-    if (cachedData)                                                         \
-      return cachedData; /* We have computed data stored on this node */    \
-                         /* in the context tree. */                         \
-    /* Else our rule node will take care of it for us. */                   \
-    return mRuleNode->GetStyle##name_(this, PR_TRUE);                       \
-  }
-#include "nsStyleStructList.h"
-#undef STYLE_STRUCT
-
-const void* nsStyleContext::PeekStyleData(nsStyleStructID aSID)
-{
-  const void* cachedData = mCachedStyleData.GetStyleData(aSID); 
-  if (cachedData)
-    return cachedData; // We have computed data stored on this node in the context tree.
-  return mRuleNode->GetStyleData(aSID, this, PR_FALSE); // Our rule node will take care of it for us.
-}
-
 // This is an evil evil function, since it forces you to alloc your own separate copy of
 // style data!  Do not use this function unless you absolutely have to!  You should avoid
 // this at all costs! -dwh
@@ -265,7 +260,7 @@ nsStyleContext::GetUniqueStyleData(const nsStyleStructID& aSID)
   const void *current = GetStyleData(aSID);
   if (!mChild && !mEmptyChild &&
       !(mBits & nsCachedStyleData::GetBitForSID(aSID)) &&
-      mCachedStyleData.GetStyleData(aSID))
+      GetCachedStyleData(aSID))
     return const_cast<void*>(current);
 
   void* result;
@@ -286,7 +281,7 @@ nsStyleContext::GetUniqueStyleData(const nsStyleStructID& aSID)
 #undef UNIQUE_CASE
 
   default:
-    NS_ERROR("Struct type not supported.  Please find another way to do this if you can!\n");
+    NS_ERROR("Struct type not supported.  Please find another way to do this if you can!");
     return nsnull;
   }
 
@@ -314,26 +309,19 @@ nsStyleContext::SetStyle(nsStyleStructID aSID, void* aStruct)
   // See the comments there (in nsRuleNode.h) for more details about
   // what this is doing and why.
 
-  const nsCachedStyleData::StyleStructInfo& info =
-      nsCachedStyleData::gInfo[aSID];
-  char* resetOrInheritSlot = reinterpret_cast<char*>(&mCachedStyleData) +
-                             info.mCachedStyleDataOffset;
-  char* resetOrInherit = reinterpret_cast<char*>
-                                         (*reinterpret_cast<void**>(resetOrInheritSlot));
-  if (!resetOrInherit) {
-    nsPresContext *presContext = mRuleNode->GetPresContext();
-    if (mCachedStyleData.IsReset(aSID)) {
-      mCachedStyleData.mResetData = new (presContext) nsResetStyleData;
-      resetOrInherit = reinterpret_cast<char*>(mCachedStyleData.mResetData);
-    } else {
-      mCachedStyleData.mInheritedData =
-          new (presContext) nsInheritedStyleData;
-      resetOrInherit =
-          reinterpret_cast<char*>(mCachedStyleData.mInheritedData);
+  void** dataSlot;
+  if (nsCachedStyleData::IsReset(aSID)) {
+    if (!mCachedResetData) {
+      mCachedResetData = new (mRuleNode->GetPresContext()) nsResetStyleData;
+      // XXXbz And if that fails?
     }
+    dataSlot = &mCachedResetData->mStyleStructs[aSID];
+  } else {
+    dataSlot = &mCachedInheritedData.mStyleStructs[aSID];
   }
-  char* dataSlot = resetOrInherit + info.mInheritResetOffset;
-  *reinterpret_cast<void**>(dataSlot) = aStruct;
+  NS_ASSERTION(!*dataSlot || (mBits & nsCachedStyleData::GetBitForSID(aSID)),
+               "Going to leak style data");
+  *dataSlot = aStruct;
 }
 
 void
@@ -351,6 +339,10 @@ nsStyleContext::ApplyStyleFixups(nsPresContext* aPresContext)
       mBits |= NS_STYLE_HAS_TEXT_DECORATIONS;
   }
 
+  if ((mParent && mParent->HasPseudoElementData()) || mPseudoTag) {
+    mBits |= NS_STYLE_HAS_PSEUDO_ELEMENT_DATA;
+  }
+
   // Correct tables.
   const nsStyleDisplay* disp = GetStyleDisplay();
   if (disp->mDisplay == NS_STYLE_DISPLAY_TABLE) {
@@ -412,19 +404,14 @@ nsStyleContext::CalcStyleDifference(nsStyleContext* aOther)
   // our position in the rule node tree is also the same.
   PRBool compare = mRuleNode != aOther->mRuleNode;
 
-  nsChangeHint maxHint = nsChangeHint(NS_STYLE_HINT_FRAMECHANGE |
-      nsChangeHint_UpdateCursor);
-  
 #define DO_STRUCT_DIFFERENCE(struct_)                                         \
   PR_BEGIN_MACRO                                                              \
     NS_ASSERTION(NS_IsHintSubset(nsStyle##struct_::MaxDifference(), maxHint), \
                  "Struct placed in the wrong maxHint section");               \
-    const nsStyle##struct_* this##struct_ =                                   \
-        static_cast<const nsStyle##struct_*>(                                 \
-                       PeekStyleData(eStyleStruct_##struct_));                \
+    const nsStyle##struct_* this##struct_ = PeekStyle##struct_();             \
     if (this##struct_) {                                                      \
       const nsStyle##struct_* other##struct_ = aOther->GetStyle##struct_();   \
-      if (compare &&                                                          \
+      if ((compare || nsStyle##struct_::ForceCompare()) &&                    \
           !NS_IsHintSubset(maxHint, hint) &&                                  \
           this##struct_ != other##struct_) {                                  \
         NS_ASSERTION(NS_IsHintSubset(                                         \
@@ -439,8 +426,13 @@ nsStyleContext::CalcStyleDifference(nsStyleContext* aOther)
   // We begin by examining those style structs that are capable of
   // causing the maximal difference, a FRAMECHANGE.
   // FRAMECHANGE Structs: Display, XUL, Content, UserInterface,
-  // Visibility, Outline, TableBorder, Table, UIReset, Quotes
+  // Visibility, Outline, TableBorder, Table, Text, UIReset, Quotes
+  nsChangeHint maxHint = nsChangeHint(NS_STYLE_HINT_FRAMECHANGE |
+      nsChangeHint_UpdateTransformLayer | nsChangeHint_UpdateOpacityLayer);
   DO_STRUCT_DIFFERENCE(Display);
+
+  maxHint = nsChangeHint(NS_STYLE_HINT_FRAMECHANGE |
+      nsChangeHint_UpdateCursor);
   DO_STRUCT_DIFFERENCE(XUL);
   DO_STRUCT_DIFFERENCE(Column);
   DO_STRUCT_DIFFERENCE(Content);
@@ -450,16 +442,15 @@ nsStyleContext::CalcStyleDifference(nsStyleContext* aOther)
   DO_STRUCT_DIFFERENCE(TableBorder);
   DO_STRUCT_DIFFERENCE(Table);
   DO_STRUCT_DIFFERENCE(UIReset);
+  DO_STRUCT_DIFFERENCE(Text);
   DO_STRUCT_DIFFERENCE(List);
   // If the quotes implementation is ever going to change we might not need
   // a framechange here and a reflow should be sufficient.  See bug 35768.
   DO_STRUCT_DIFFERENCE(Quotes);
 
-#ifdef MOZ_SVG
   maxHint = nsChangeHint(NS_STYLE_HINT_REFLOW | nsChangeHint_UpdateEffects);
   DO_STRUCT_DIFFERENCE(SVGReset);
   DO_STRUCT_DIFFERENCE(SVG);
-#endif
 
   // At this point, we know that the worst kind of damage we could do is
   // a reflow.
@@ -473,20 +464,122 @@ nsStyleContext::CalcStyleDifference(nsStyleContext* aOther)
   DO_STRUCT_DIFFERENCE(Padding);
   DO_STRUCT_DIFFERENCE(Border);
   DO_STRUCT_DIFFERENCE(Position);
-  DO_STRUCT_DIFFERENCE(Text);
   DO_STRUCT_DIFFERENCE(TextReset);
 
-  // At this point, we know that the worst kind of damage we could do is
-  // a re-render (i.e., a VISUAL change).
-  maxHint = NS_STYLE_HINT_VISUAL;
-
-  // The following structs cause (as their maximal difference) a
-  // re-render to occur.  VISUAL Structs: Color, Background
-  DO_STRUCT_DIFFERENCE(Color);
+  // Most backgrounds only require a re-render (i.e., a VISUAL change), but
+  // backgrounds using -moz-element need to reset SVG effects, too.
+  maxHint = nsChangeHint(NS_STYLE_HINT_VISUAL | nsChangeHint_UpdateEffects);
   DO_STRUCT_DIFFERENCE(Background);
 
+  // Color only needs a repaint.
+  maxHint = NS_STYLE_HINT_VISUAL;
+  DO_STRUCT_DIFFERENCE(Color);
+
 #undef DO_STRUCT_DIFFERENCE
 
+  // Note that we do not check whether this->RelevantLinkVisited() !=
+  // aOther->RelevantLinkVisited(); we don't need to since
+  // nsCSSFrameConstructor::DoContentStateChanged always adds
+  // nsChangeHint_RepaintFrame for NS_EVENT_STATE_VISITED changes (and
+  // needs to, since HasStateDependentStyle probably doesn't work right
+  // for NS_EVENT_STATE_VISITED).  Hopefully this doesn't actually
+  // expose whether links are visited to performance tests since all
+  // link coloring happens asynchronously at a time when it's hard for
+  // the page to measure.
+  // However, we do need to compute the larger of the changes that can
+  // happen depending on whether the link is visited or unvisited, since
+  // doing only the one that's currently appropriate would expose which
+  // links are in history to easy performance measurement.  Therefore,
+  // here, we add nsChangeHint_RepaintFrame hints (the maximum for
+  // things that can depend on :visited) for the properties on which we
+  // call GetVisitedDependentColor.
+  nsStyleContext *thisVis = GetStyleIfVisited(),
+                *otherVis = aOther->GetStyleIfVisited();
+  if (!thisVis != !otherVis) {
+    // One style context has a style-if-visited and the other doesn't.
+    // Presume a difference.
+    NS_UpdateHint(hint, nsChangeHint_RepaintFrame);
+  } else if (thisVis && !NS_IsHintSubset(nsChangeHint_RepaintFrame, hint)) {
+    // Both style contexts have a style-if-visited.
+    PRBool change = PR_FALSE;
+
+    // NB: Calling Peek on |this|, not |thisVis|, since callers may look
+    // at a struct on |this| without looking at the same struct on
+    // |thisVis| (including this function if we skip one of these checks
+    // due to change being true already or due to the old style context
+    // not having a style-if-visited), but not the other way around.
+    if (PeekStyleColor()) {
+      if (thisVis->GetStyleColor()->mColor !=
+          otherVis->GetStyleColor()->mColor) {
+        change = PR_TRUE;
+      }
+    }
+
+    // NB: Calling Peek on |this|, not |thisVis| (see above).
+    if (!change && PeekStyleBackground()) {
+      if (thisVis->GetStyleBackground()->mBackgroundColor !=
+          otherVis->GetStyleBackground()->mBackgroundColor) {
+        change = PR_TRUE;
+      }
+    }
+
+    // NB: Calling Peek on |this|, not |thisVis| (see above).
+    if (!change && PeekStyleBorder()) {
+      const nsStyleBorder *thisVisBorder = thisVis->GetStyleBorder();
+      const nsStyleBorder *otherVisBorder = otherVis->GetStyleBorder();
+      NS_FOR_CSS_SIDES(side) {
+        PRBool thisFG, otherFG;
+        nscolor thisColor, otherColor;
+        thisVisBorder->GetBorderColor(side, thisColor, thisFG);
+        otherVisBorder->GetBorderColor(side, otherColor, otherFG);
+        if (thisFG != otherFG || (!thisFG && thisColor != otherColor)) {
+          change = PR_TRUE;
+          break;
+        }
+      }
+    }
+
+    // NB: Calling Peek on |this|, not |thisVis| (see above).
+    if (!change && PeekStyleOutline()) {
+      const nsStyleOutline *thisVisOutline = thisVis->GetStyleOutline();
+      const nsStyleOutline *otherVisOutline = otherVis->GetStyleOutline();
+      PRBool haveColor;
+      nscolor thisColor, otherColor;
+      if (thisVisOutline->GetOutlineInitialColor() != 
+            otherVisOutline->GetOutlineInitialColor() ||
+          (haveColor = thisVisOutline->GetOutlineColor(thisColor)) != 
+            otherVisOutline->GetOutlineColor(otherColor) ||
+          (haveColor && thisColor != otherColor)) {
+        change = PR_TRUE;
+      }
+    }
+
+    // NB: Calling Peek on |this|, not |thisVis| (see above).
+    if (!change && PeekStyleColumn()) {
+      const nsStyleColumn *thisVisColumn = thisVis->GetStyleColumn();
+      const nsStyleColumn *otherVisColumn = otherVis->GetStyleColumn();
+      if (thisVisColumn->mColumnRuleColor != otherVisColumn->mColumnRuleColor ||
+          thisVisColumn->mColumnRuleColorIsForeground !=
+            otherVisColumn->mColumnRuleColorIsForeground) {
+        change = PR_TRUE;
+      }
+    }
+
+    // NB: Calling Peek on |this|, not |thisVis| (see above).
+    if (!change && PeekStyleSVG()) {
+      const nsStyleSVG *thisVisSVG = thisVis->GetStyleSVG();
+      const nsStyleSVG *otherVisSVG = otherVis->GetStyleSVG();
+      if (thisVisSVG->mFill != otherVisSVG->mFill ||
+          thisVisSVG->mStroke != otherVisSVG->mStroke) {
+        change = PR_TRUE;
+      }
+    }
+
+    if (change) {
+      NS_UpdateHint(hint, nsChangeHint_RepaintFrame);
+    }
+  }
+
   return hint;
 }
 
@@ -515,44 +608,6 @@ nsStyleContext::Mark()
 }
 
 #ifdef DEBUG
-
-class URICString : public nsCAutoString {
-public:
-  URICString(nsIURI* aURI) {
-    if (aURI) {
-      aURI->GetSpec(*this);
-    } else {
-      Assign("[none]");
-    }
-  }
-
-  URICString(imgIRequest* aImageRequest) {
-    nsCOMPtr<nsIURI> uri;
-    if (aImageRequest) {
-      aImageRequest->GetURI(getter_AddRefs(uri));
-    }
-    if (uri) {
-      uri->GetSpec(*this);
-    } else {
-      Assign("[none]");
-    }
-  }
-
-  URICString(nsCSSValue::URL* aURI) {
-    if (aURI) {
-      NS_ASSERTION(aURI->mURI, "Must have URI here!");
-      aURI->mURI->GetSpec(*this);
-    } else {
-      Assign("[none]");
-    }
-  }
-  
-  URICString& operator=(const URICString& aOther) {
-    Assign(aOther);
-    return *this;
-  }
-};
-
 void nsStyleContext::List(FILE* out, PRInt32 aIndent)
 {
   // Indent
@@ -599,324 +654,6 @@ void nsStyleContext::List(FILE* out, PRInt32 aIndent)
     } while (mEmptyChild != child);
   }
 }
-
-static void IndentBy(FILE* out, PRInt32 aIndent) {
-  while (--aIndent >= 0) fputs("  ", out);
-}
-// virtual 
-void nsStyleContext::DumpRegressionData(nsPresContext* aPresContext, FILE* out, PRInt32 aIndent)
-{
-  nsAutoString str;
-
-  // FONT
-  IndentBy(out,aIndent);
-  const nsStyleFont* font = GetStyleFont();
-  fprintf(out, "<font %s %d %d %d />\n", 
-          NS_ConvertUTF16toUTF8(font->mFont.name).get(),
-          font->mFont.size,
-          font->mSize,
-          font->mGenericID);
-
-  // COLOR
-  IndentBy(out,aIndent);
-  const nsStyleColor* color = GetStyleColor();
-  fprintf(out, "<color data=\"%ld\"/>\n", 
-    (long)color->mColor);
-
-  // BACKGROUND
-  IndentBy(out,aIndent);
-  const nsStyleBackground* bg = GetStyleBackground();
-  fprintf(out, "<background data=\"%d %d %d %ld %ld %ld %s\"/>\n",
-    (int)bg->mBackgroundAttachment,
-    (int)bg->mBackgroundFlags,
-    (int)bg->mBackgroundRepeat,
-    (long)bg->mBackgroundColor,
-    // XXX These aren't initialized unless flags are set:
-    (long)bg->mBackgroundXPosition.mCoord, // potentially lossy on some platforms
-    (long)bg->mBackgroundYPosition.mCoord, // potentially lossy on some platforms
-    URICString(bg->mBackgroundImage).get());
- 
-  // SPACING (ie. margin, padding, border, outline)
-  IndentBy(out,aIndent);
-  fprintf(out, "<spacing data=\"");
-
-  const nsStyleMargin* margin = GetStyleMargin();
-  margin->mMargin.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  
-  const nsStylePadding* padding = GetStylePadding();
-  padding->mPadding.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  
-  const nsStyleBorder* border = GetStyleBorder();
-#ifdef NS_COORD_IS_FLOAT
-  const char format [] = "top: %ftw right: %ftw bottom: %ftw left: %ftw";
-#else
-  const char format [] = "top: %dtw right: %dtw bottom: %dtw left: %dtw";
-#endif
-  nsPrintfCString output(format,
-                         border->GetActualBorderWidth(NS_SIDE_TOP),
-                         border->GetActualBorderWidth(NS_SIDE_RIGHT),
-                         border->GetActualBorderWidth(NS_SIDE_BOTTOM),
-                         border->GetActualBorderWidth(NS_SIDE_LEFT));
-  fprintf(out, "%s ", output.get());
-  border->mBorderRadius.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  
-  const nsStyleOutline* outline = GetStyleOutline();
-  outline->mOutlineRadius.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  outline->mOutlineWidth.ToString(str);
-  fprintf(out, "%s", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "%d", (int)border->mFloatEdge);
-  fprintf(out, "\" />\n");
-
-  // LIST
-  IndentBy(out,aIndent);
-  const nsStyleList* list = GetStyleList();
-  fprintf(out, "<list data=\"%d %d %s\" />\n",
-    (int)list->mListStyleType,
-    (int)list->mListStyleType,
-    URICString(list->mListStyleImage).get());
-
-  // POSITION
-  IndentBy(out,aIndent);
-  const nsStylePosition* pos = GetStylePosition();
-  fprintf(out, "<position data=\"");
-  pos->mOffset.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  pos->mWidth.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  pos->mMinWidth.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  pos->mMaxWidth.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  pos->mHeight.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  pos->mMinHeight.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  pos->mMaxHeight.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "%d ", (int)pos->mBoxSizing);
-  pos->mZIndex.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "\" />\n");
-
-  // TEXT
-  IndentBy(out,aIndent);
-  const nsStyleText* text = GetStyleText();
-  fprintf(out, "<text data=\"%d %d %d %d",
-    (int)text->mTextAlign,
-    (int)text->mTextTransform,
-    (int)text->mWhiteSpace,
-    (int)text->mWordWrap);
-  text->mLetterSpacing.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  text->mLineHeight.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  text->mTextIndent.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  text->mWordSpacing.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "\" />\n");
-  
-  // TEXT RESET
-  IndentBy(out,aIndent);
-  const nsStyleTextReset* textReset = GetStyleTextReset();
-  fprintf(out, "<textreset data=\"%d ",
-    (int)textReset->mTextDecoration);
-  textReset->mVerticalAlign.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "\" />\n");
-
-  // DISPLAY
-  IndentBy(out,aIndent);
-  const nsStyleDisplay* disp = GetStyleDisplay();
-  fprintf(out, "<display data=\"%d %d %f %d %d %d %d %d %d %d %ld %ld %ld %ld %s\" />\n",
-    (int)disp->mPosition,
-    (int)disp->mDisplay,
-    (float)disp->mOpacity,      
-    (int)disp->mFloats,
-    (int)disp->mBreakType,
-    (int)disp->mBreakBefore,
-    (int)disp->mBreakAfter,
-    (int)disp->mOverflowX,
-    (int)disp->mOverflowY,
-    (int)disp->mClipFlags,
-    (long)disp->mClip.x,
-    (long)disp->mClip.y,
-    (long)disp->mClip.width,
-    (long)disp->mClip.height,
-    URICString(disp->mBinding).get()
-    );
-  
-  // VISIBILITY
-  IndentBy(out,aIndent);
-  const nsStyleVisibility* vis = GetStyleVisibility();
-  fprintf(out, "<visibility data=\"%d %d\" />\n",
-    (int)vis->mDirection,
-    (int)vis->mVisible
-    );
-
-  // TABLE
-  IndentBy(out,aIndent);
-  const nsStyleTable* table = GetStyleTable();
-  fprintf(out, "<table data=\"%d %d %d ",
-    (int)table->mLayoutStrategy,
-    (int)table->mFrame,
-    (int)table->mRules);
-  fprintf(out, "%ld %ld ",
-    (long)table->mCols,
-    (long)table->mSpan);
-  fprintf(out, "\" />\n");
-
-  // TABLEBORDER
-  IndentBy(out,aIndent);
-  const nsStyleTableBorder* tableBorder = GetStyleTableBorder();
-  fprintf(out, "<tableborder data=\"%d %d %d %d %d ",
-    (int)tableBorder->mBorderCollapse,
-    (int)tableBorder->mBorderSpacingX,
-    (int)tableBorder->mBorderSpacingY,
-    (int)tableBorder->mCaptionSide,
-    (int)tableBorder->mEmptyCells);
-  fprintf(out, "\" />\n");
-
-  // CONTENT
-  IndentBy(out,aIndent);
-  const nsStyleContent* content = GetStyleContent();
-  fprintf(out, "<content data=\"%ld %ld %ld ",
-    (long)content->ContentCount(),
-    (long)content->CounterIncrementCount(),
-    (long)content->CounterResetCount());
-  // XXX: iterate over the content and counters...
-  content->mMarkerOffset.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "\" />\n");
-
-  // QUOTES
-  IndentBy(out,aIndent);
-  const nsStyleQuotes* quotes = GetStyleQuotes();
-  fprintf(out, "<quotes data=\"%ld ",
-    (long)quotes->QuotesCount());
-  // XXX: iterate over the quotes...
-  fprintf(out, "\" />\n");
-
-  // UI
-  IndentBy(out,aIndent);
-  const nsStyleUserInterface* ui = GetStyleUserInterface();
-  fprintf(out, "<ui data=\"%d %d %d %d\" />\n",
-    (int)ui->mUserInput,
-    (int)ui->mUserModify,
-    (int)ui->mUserFocus, 
-    (int)ui->mCursor);
-
-  // UIReset
-  IndentBy(out,aIndent);
-  const nsStyleUIReset* uiReset = GetStyleUIReset();
-  fprintf(out, "<uireset data=\"%d %d %d\" />\n",
-    (int)uiReset->mUserSelect,
-    (int)uiReset->mIMEMode,
-    (int)uiReset->mWindowShadow);
-
-  // Column
-  IndentBy(out,aIndent);
-  const nsStyleColumn* column = GetStyleColumn();
-  fprintf(out, "<column data=\"%d ",
-    (int)column->mColumnCount);
-  column->mColumnWidth.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  column->mColumnGap.ToString(str);
-  fprintf(out, "%s ", NS_ConvertUTF16toUTF8(str).get());
-  fprintf(out, "%d %d %ld",
-    (int)column->GetComputedColumnRuleWidth(),
-    (int)column->mColumnRuleStyle,
-    (long)column->mColumnRuleColor);
-  fprintf(out, "\" />\n");
-
-  // XUL
-  IndentBy(out,aIndent);
-  const nsStyleXUL* xul = GetStyleXUL();
-  fprintf(out, "<xul data=\"%d %d %d %d %d %d",
-    (int)xul->mBoxAlign,
-    (int)xul->mBoxDirection,
-    (int)xul->mBoxFlex,
-    (int)xul->mBoxOrient,
-    (int)xul->mBoxPack,
-    (int)xul->mBoxOrdinal);
-  fprintf(out, "\" />\n");
-
-#ifdef MOZ_SVG
-  // SVG
-  IndentBy(out,aIndent);
-  const nsStyleSVG* svg = GetStyleSVG();
-  fprintf(out, "<svg data=\"%d ",(int)svg->mFill.mType);
-  if (svg->mFill.mType == eStyleSVGPaintType_Server)
-    fprintf(out, "%s %ld ", URICString(svg->mFill.mPaint.mPaintServer).get(),
-                            (long)svg->mFill.mFallbackColor);
-  else
-    fprintf(out, "%ld ", (long)svg->mFill.mPaint.mColor);
-
-  fprintf(out, "%d ", (int)svg->mStroke.mType);
-  if (svg->mStroke.mType == eStyleSVGPaintType_Server)
-    fprintf(out, "%s %ld ", URICString(svg->mStroke.mPaint.mPaintServer).get(),
-                            (long)svg->mStroke.mFallbackColor);
-  else
-    fprintf(out, "%ld ", (long)svg->mStroke.mPaint.mColor);
-
-  fprintf(out, "%s %s %s ",
-          URICString(svg->mMarkerEnd).get(),
-          URICString(svg->mMarkerMid).get(),
-          URICString(svg->mMarkerStart).get());
-
-  for (PRUint32 i = 0; i < svg->mStrokeDasharrayLength; i++) {
-    svg->mStrokeDasharray[i].ToString(str);
-    fprintf(out,
-            "%s%c",
-            NS_ConvertUTF16toUTF8(str).get(),
-            (i == svg->mStrokeDasharrayLength) ? ' ' : ',');
-  }
-
-  svg->mStrokeDashoffset.ToString(str);
-  fprintf(out, "%f %s %f %f ",
-          svg->mFillOpacity,
-          NS_ConvertUTF16toUTF8(str).get(),
-          svg->mStrokeMiterlimit,
-          svg->mStrokeOpacity);
-  svg->mStrokeWidth.ToString(str);
-  fprintf(out, "%s %d %d %d %d %d %d %d %d %d %d %d\" />\n",
-          NS_ConvertUTF16toUTF8(str).get(),
-          (int)svg->mStrokeDasharrayLength,
-          (int)svg->mClipRule,
-          (int)svg->mColorInterpolation,
-          (int)svg->mColorInterpolationFilters,
-          (int)svg->mFillRule,
-          (int)svg->mPointerEvents,
-          (int)svg->mShapeRendering,
-          (int)svg->mStrokeLinecap,
-          (int)svg->mStrokeLinejoin,
-          (int)svg->mTextAnchor,
-          (int)svg->mTextRendering);
-
-  // SVGReset
-  IndentBy(out,aIndent);
-  const nsStyleSVGReset* svgReset = GetStyleSVGReset();
-
-  fprintf(out, "<svgreset data=\"%ld ", (long)svgReset->mStopColor);
-
-  fprintf(out, "%ld ", (long)svgReset->mFloodColor);
-
-  fprintf(out, "%ld ", (long)svgReset->mLightingColor);
-
-  fprintf(out, "%s %s %s %f %f %d\" />\n",
-          URICString(svgReset->mClipPath).get(),
-          URICString(svgReset->mFilter).get(),
-          URICString(svgReset->mMask).get(),
-          svgReset->mStopOpacity,
-          svgReset->mFloodOpacity,
-          (int)svgReset->mDominantBaseline);
-#endif
-  //#insert new style structs here#
-}
 #endif
 
 // Overloaded new operator. Initializes the memory to 0 and relies on an arena
@@ -934,7 +671,7 @@ void
 nsStyleContext::Destroy()
 {
   // Get the pres context from our rule node.
-  nsCOMPtr<nsPresContext> presContext = mRuleNode->GetPresContext();
+  nsRefPtr<nsPresContext> presContext = mRuleNode->GetPresContext();
 
   // Call our destructor.
   this->~nsStyleContext();
@@ -947,13 +684,112 @@ nsStyleContext::Destroy()
 already_AddRefed<nsStyleContext>
 NS_NewStyleContext(nsStyleContext* aParentContext,
                    nsIAtom* aPseudoTag,
+                   nsCSSPseudoElements::Type aPseudoType,
                    nsRuleNode* aRuleNode,
                    nsPresContext* aPresContext)
 {
-  nsStyleContext* context = new (aPresContext) nsStyleContext(aParentContext, aPseudoTag, 
-                                                              aRuleNode, aPresContext);
+  nsStyleContext* context =
+    new (aPresContext) nsStyleContext(aParentContext, aPseudoTag, aPseudoType,
+                                      aRuleNode, aPresContext);
   if (context)
     context->AddRef();
   return context;
 }
 
+static nscolor ExtractColor(nsCSSProperty aProperty,
+                            nsStyleContext *aStyleContext)
+{
+  nsStyleAnimation::Value val;
+#ifdef DEBUG
+  PRBool success =
+#endif
+    nsStyleAnimation::ExtractComputedValue(aProperty, aStyleContext, val);
+  NS_ABORT_IF_FALSE(success,
+                    "aProperty must be extractable by nsStyleAnimation");
+  return val.GetColorValue();
+}
+
+struct ColorIndexSet {
+  PRUint8 colorIndex, alphaIndex;
+};
+
+static const ColorIndexSet gVisitedIndices[2] = { { 0, 0 }, { 1, 0 } };
+
+nscolor
+nsStyleContext::GetVisitedDependentColor(nsCSSProperty aProperty)
+{
+  NS_ASSERTION(aProperty == eCSSProperty_color ||
+               aProperty == eCSSProperty_background_color ||
+               aProperty == eCSSProperty_border_top_color ||
+               aProperty == eCSSProperty_border_right_color_value ||
+               aProperty == eCSSProperty_border_bottom_color ||
+               aProperty == eCSSProperty_border_left_color_value ||
+               aProperty == eCSSProperty_outline_color ||
+               aProperty == eCSSProperty__moz_column_rule_color ||
+               aProperty == eCSSProperty_fill ||
+               aProperty == eCSSProperty_stroke,
+               "we need to add to nsStyleContext::CalcStyleDifference");
+
+  nscolor colors[2];
+  colors[0] = ExtractColor(aProperty, this);
+
+  nsStyleContext *visitedStyle = this->GetStyleIfVisited();
+  if (!visitedStyle) {
+    return colors[0];
+  }
+
+  colors[1] = ExtractColor(aProperty, visitedStyle);
+
+  return nsStyleContext::CombineVisitedColors(colors,
+                                              this->RelevantLinkVisited());
+}
+
+/* static */ nscolor
+nsStyleContext::CombineVisitedColors(nscolor *aColors, PRBool aLinkIsVisited)
+{
+  if (NS_GET_A(aColors[1]) == 0) {
+    // If the style-if-visited is transparent, then just use the
+    // unvisited style rather than using the (meaningless) color
+    // components of the visited style along with a potentially
+    // non-transparent alpha value.
+    aLinkIsVisited = PR_FALSE;
+  }
+
+  // NOTE: We want this code to have as little timing dependence as
+  // possible on whether this->RelevantLinkVisited() is true.
+  const ColorIndexSet &set =
+    gVisitedIndices[aLinkIsVisited ? 1 : 0];
+
+  nscolor colorColor = aColors[set.colorIndex];
+  nscolor alphaColor = aColors[set.alphaIndex];
+  return NS_RGBA(NS_GET_R(colorColor), NS_GET_G(colorColor),
+                 NS_GET_B(colorColor), NS_GET_A(alphaColor));
+}
+
+void*
+nsStyleContext::Alloc(size_t aSize)
+{
+  nsIPresShell *shell = PresContext()->PresShell();
+
+  aSize += offsetof(AllocationHeader, mStorageStart);
+  AllocationHeader *alloc =
+    static_cast<AllocationHeader*>(shell->AllocateMisc(aSize));
+
+  alloc->mSize = aSize; // NOTE: inflated by header
+
+  alloc->mNext = mAllocations;
+  mAllocations = alloc;
+
+  return static_cast<void*>(&alloc->mStorageStart);
+}
+
+void
+nsStyleContext::FreeAllocations(nsPresContext *aPresContext)
+{
+  nsIPresShell *shell = aPresContext->PresShell();
+
+  for (AllocationHeader *alloc = mAllocations, *next; alloc; alloc = next) {
+    next = alloc->mNext;
+    shell->FreeMisc(alloc->mSize, alloc);
+  }
+}
