@@ -43,7 +43,6 @@
 #include "nsILineIterator.h"
 #include "nsTablePainter.h"
 #include "nsTArray.h"
-#include "nsCSSAnonBoxes.h"
 
 class nsTableFrame;
 class nsTableRowFrame;
@@ -72,18 +71,14 @@ struct nsRowGroupReflowState {
   ~nsRowGroupReflowState() {}
 };
 
-#define NS_ITABLEROWGROUPFRAME_IID    \
-{ 0xe940e7bc, 0xb534, 0x11d2,  \
-  { 0x95, 0xa2, 0x0, 0x60, 0xb0, 0xc3, 0x44, 0x14 } }
-
 // use the following bits from nsFrame's frame state 
 
 // thead or tfoot should be repeated on every printed page
-#define NS_ROWGROUP_REPEATABLE           0x80000000
-#define NS_ROWGROUP_HAS_STYLE_HEIGHT     0x40000000
+#define NS_ROWGROUP_REPEATABLE           NS_FRAME_STATE_BIT(31)
+#define NS_ROWGROUP_HAS_STYLE_HEIGHT     NS_FRAME_STATE_BIT(30)
 // the next is also used on rows (see nsTableRowGroupFrame::InitRepeatedFrame)
-#define NS_REPEATED_ROW_OR_ROWGROUP      0x10000000
-#define NS_ROWGROUP_HAS_ROW_CURSOR       0x08000000
+#define NS_REPEATED_ROW_OR_ROWGROUP      NS_FRAME_STATE_BIT(28)
+#define NS_ROWGROUP_HAS_ROW_CURSOR       NS_FRAME_STATE_BIT(27)
 
 #define MIN_ROWS_NEEDING_CURSOR 20
 
@@ -101,7 +96,9 @@ class nsTableRowGroupFrame
   , public nsILineIterator
 {
 public:
-  NS_IMETHOD QueryInterface(const nsIID &aIID, void **aInstancePtr);
+  NS_DECL_QUERYFRAME_TARGET(nsTableRowGroupFrame)
+  NS_DECL_QUERYFRAME
+  NS_DECL_FRAMEARENA_HELPERS
 
   /** instantiate a new instance of nsTableRowFrame.
     * @param aPresShell the pres shell for this frame
@@ -114,11 +111,11 @@ public:
   virtual void DidSetStyleContext(nsStyleContext* aOldStyleContext);
   
   NS_IMETHOD AppendFrames(nsIAtom*        aListName,
-                          nsIFrame*       aFrameList);
+                          nsFrameList&    aFrameList);
   
   NS_IMETHOD InsertFrames(nsIAtom*        aListName,
                           nsIFrame*       aPrevFrame,
-                          nsIFrame*       aFrameList);
+                          nsFrameList&    aFrameList);
 
   NS_IMETHOD RemoveFrame(nsIAtom*        aListName,
                          nsIFrame*       aOldFrame);
@@ -145,12 +142,6 @@ public:
                     const nsHTMLReflowState& aReflowState,
                     nsReflowStatus&          aStatus);
 
-  virtual PRBool IsFrameOfType(PRUint32 aFlags) const
-  {
-    return nsHTMLContainerFrame::IsFrameOfType(aFlags &
-      ~nsIFrame::eExcludesIgnorableWhitespace);
-  }
-
   /**
    * Get the "type" of the frame
    *
@@ -271,11 +262,6 @@ public:
     *                       frame. -1 if the frame cannot be found.
     */
   virtual PRInt32 FindLineContaining(nsIFrame* aFrame);
-  
-  /** not implemented
-    * the function is also not called in our tree
-    */
-  virtual PRInt32 FindLineAt(nscoord aY);
 
   /** Find the orginating cell frame on a row that is the nearest to the
     * coordinate X.
@@ -367,13 +353,6 @@ public:
    * decided not to use a cursor or we already have one set up.
    */
   FrameCursorData* SetupRowCursor();
-  
-  PRBool IsScrolled() {
-    // Note that if mOverflowY is CLIP, so is mOverflowX, and we need to clip the background
-    // as if the rowgroup is scrollable.
-    return GetStyleContext()->GetPseudoType() == nsCSSAnonBoxes::scrolledContent ||
-           GetStyleDisplay()->mOverflowY == NS_STYLE_OVERFLOW_CLIP;
-  }
 
   virtual nsILineIterator* GetLineIterator() { return this; }
 
@@ -392,7 +371,7 @@ protected:
                   nsIFrame*              aKidFrame,
                   nsHTMLReflowMetrics&   aDesiredSize,
                   const nsRect&          aOriginalKidRect,
-                  const nsRect&          aOriginalKidOverflowRect);
+                  const nsRect&          aOriginalKidVisualOverflow);
 
   void CalculateRowHeights(nsPresContext*           aPresContext, 
                            nsHTMLReflowMetrics&     aDesiredSize,
@@ -411,11 +390,11 @@ protected:
    * @return  true if we successfully reflowed all the mapped children and false
    *            otherwise, e.g. we pushed children to the next in flow
    */
-  NS_METHOD ReflowChildren(nsPresContext*         aPresContext,
-                           nsHTMLReflowMetrics&   aDesiredSize,
-                           nsRowGroupReflowState& aReflowState,
-                           nsReflowStatus&        aStatus,
-                           PRBool*                aPageBreakBeforeEnd = nsnull);
+  nsresult ReflowChildren(nsPresContext*         aPresContext,
+                          nsHTMLReflowMetrics&   aDesiredSize,
+                          nsRowGroupReflowState& aReflowState,
+                          nsReflowStatus&        aStatus,
+                          PRBool*                aPageBreakBeforeEnd = nsnull);
 
   nsresult SplitRowGroup(nsPresContext*           aPresContext,
                          nsHTMLReflowMetrics&     aDesiredSize,
@@ -453,14 +432,12 @@ private:
   BCPixelSize mLeftContBorderWidth;
 
 public:
-  virtual nsIFrame* GetFirstFrame() { return mFrames.FirstChild(); }
-  virtual nsIFrame* GetLastFrame() { return mFrames.LastChild(); }
-  virtual void GetNextFrame(nsIFrame*  aFrame, 
-                            nsIFrame** aResult) { *aResult = aFrame->GetNextSibling(); }
   PRBool IsRepeatable() const;
   void   SetRepeatable(PRBool aRepeatable);
   PRBool HasStyleHeight() const;
   void   SetHasStyleHeight(PRBool aValue);
+  PRBool HasInternalBreakBefore() const;
+  PRBool HasInternalBreakAfter() const;
 };
 
 
