@@ -44,6 +44,7 @@ Runs the reftest test harness.
 import sys, shutil, os, os.path
 SCRIPT_DIRECTORY = os.path.abspath(os.path.realpath(os.path.dirname(sys.argv[0])))
 sys.path.append(SCRIPT_DIRECTORY)
+
 from automation import Automation
 from automationutils import *
 from optparse import OptionParser
@@ -55,21 +56,33 @@ class RefTest(object):
 
   def __init__(self, automation):
     self.automation = automation
-    os.chdir(SCRIPT_DIRECTORY)
 
   def getFullPath(self, path):
     "Get an absolute path relative to self.oldcwd."
     return os.path.normpath(os.path.join(self.oldcwd, os.path.expanduser(path)))
 
-  def createReftestProfile(self, options, profileDir):
+  def getManifestPath(self, path):
+    "Get the path of the manifest, and for remote testing this function is subclassed to point to remote manifest"
+    return self.getFullPath(path)
+
+  def createReftestProfile(self, options, profileDir, server='localhost'):
     "Sets up a profile for reftest."
 
-    # Set preferences.
-    prefsFile = open(os.path.join(profileDir, "user.js"), "w")
-    prefsFile.write("""user_pref("browser.dom.window.dump.enabled", true);
-    """)
+    self.automation.setupPermissionsDatabase(profileDir,
+      {'allowXULXBL': [(server, True), ('<file>', True)]})
+
+    # Set preferences for communication between our command line arguments
+    # and the reftest harness.  Preferences that are required for reftest
+    # to work should instead be set in reftest-cmdline.js .
+    prefsFile = open(os.path.join(profileDir, "user.js"), "a")
     prefsFile.write('user_pref("reftest.timeout", %d);\n' % (options.timeout * 1000))
-    prefsFile.write('user_pref("ui.caretBlinkTime", -1);\n')
+
+    if options.totalChunks != None:
+      prefsFile.write('user_pref("reftest.totalChunks", %d);\n' % options.totalChunks)
+    if options.thisChunk != None:
+      prefsFile.write('user_pref("reftest.thisChunk", %d);\n' % options.thisChunk)
+    if options.logFile != None:
+      prefsFile.write('user_pref("reftest.logFile", "%s");\n' % options.logFile)
 
     for v in options.extraPrefs:
       thispref = v.split("=")
@@ -78,18 +91,44 @@ class RefTest(object):
         sys.exit(1)
       part = 'user_pref("%s", %s);\n' % (thispref[0], thispref[1])
       prefsFile.write(part)
-    # no slow script dialogs
-    prefsFile.write('user_pref("dom.max_script_run_time", 0);')
-    prefsFile.write('user_pref("dom.max_chrome_script_run_time", 0);')
     prefsFile.close()
 
     # install the reftest extension bits into the profile
-    profileExtensionsPath = os.path.join(profileDir, "extensions")
-    os.mkdir(profileExtensionsPath)
-    reftestExtensionPath = os.path.join(SCRIPT_DIRECTORY, "reftest")
-    extFile = open(os.path.join(profileExtensionsPath, "reftest@mozilla.org"), "w")
-    extFile.write(reftestExtensionPath)
-    extFile.close()
+    self.automation.installExtension(os.path.join(SCRIPT_DIRECTORY, "reftest"),
+                                                  profileDir,
+                                                  "reftest@mozilla.org")
+
+
+  def registerExtension(self, browserEnv, options, profileDir, extraArgs = ['-silent']):
+    # run once with -silent to let the extension manager do its thing
+    # and then exit the app
+    self.automation.log.info("REFTEST INFO | runreftest.py | Performing extension manager registration: start.\n")
+    # Don't care about this |status|: |runApp()| reporting it should be enough.
+    status = self.automation.runApp(None, browserEnv, options.app, profileDir,
+                                 extraArgs,
+                                 utilityPath = options.utilityPath,
+                                 xrePath=options.xrePath,
+                                 symbolsPath=options.symbolsPath)
+    # We don't care to call |processLeakLog()| for this step.
+    self.automation.log.info("\nREFTEST INFO | runreftest.py | Performing extension manager registration: end.")
+
+    # Remove the leak detection file so it can't "leak" to the tests run.
+    # The file is not there if leak logging was not enabled in the application build.
+    if os.path.exists(self.leakLogFile):
+      os.remove(self.leakLogFile)
+
+  def buildBrowserEnv(self, options, profileDir):
+    browserEnv = self.automation.environment(xrePath = options.xrePath)
+    browserEnv["XPCOM_DEBUG_BREAK"] = "stack"
+
+    # Enable leaks detection to its own log file.
+    self.leakLogFile = os.path.join(profileDir, "runreftest_leaks.log")
+    browserEnv["XPCOM_MEM_BLOAT_LOG"] = self.leakLogFile
+    return browserEnv
+
+  def cleanup(self, profileDir):
+    if profileDir:
+      shutil.rmtree(profileDir)
 
   def runTests(self, manifest, options):
     debuggerInfo = getDebuggerInfo(self.oldcwd, options.debugger, options.debuggerArgs,
@@ -98,37 +137,18 @@ class RefTest(object):
     profileDir = None
     try:
       profileDir = mkdtemp()
-      self.createReftestProfile(options, profileDir)
       self.copyExtraFilesToProfile(options, profileDir)
+      self.createReftestProfile(options, profileDir)
+      self.installExtensionsToProfile(options, profileDir)
 
       # browser environment
-      browserEnv = self.automation.environment(xrePath = options.xrePath)
-      browserEnv["XPCOM_DEBUG_BREAK"] = "stack"
+      browserEnv = self.buildBrowserEnv(options, profileDir)
 
-      # Enable leaks detection to its own log file.
-      leakLogFile = os.path.join(profileDir, "runreftest_leaks.log")
-      browserEnv["XPCOM_MEM_BLOAT_LOG"] = leakLogFile
-
-      # run once with -silent to let the extension manager do its thing
-      # and then exit the app
-      self.automation.log.info("REFTEST INFO | runreftest.py | Performing extension manager registration: start.\n")
-      # Don't care about this |status|: |runApp()| reporting it should be enough.
-      status = self.automation.runApp(None, browserEnv, options.app, profileDir,
-                                 ["-silent"],
-                                 utilityPath = options.utilityPath,
-                                 xrePath=options.xrePath,
-                                 symbolsPath=options.symbolsPath)
-      # We don't care to call |processLeakLog()| for this step.
-      self.automation.log.info("\nREFTEST INFO | runreftest.py | Performing extension manager registration: end.")
-
-      # Remove the leak detection file so it can't "leak" to the tests run.
-      # The file is not there if leak logging was not enabled in the application build.
-      if os.path.exists(leakLogFile):
-        os.remove(leakLogFile)
+      self.registerExtension(browserEnv, options, profileDir)
 
       # then again to actually run reftest
       self.automation.log.info("REFTEST INFO | runreftest.py | Running tests: start.\n")
-      reftestlist = self.getFullPath(manifest)
+      reftestlist = self.getManifestPath(manifest)
       status = self.automation.runApp(None, browserEnv, options.app, profileDir,
                                  ["-reftest", reftestlist],
                                  utilityPath = options.utilityPath,
@@ -138,11 +158,10 @@ class RefTest(object):
                                  # give the JS harness 30 seconds to deal
                                  # with its own timeouts
                                  timeout=options.timeout + 30.0)
-      processLeakLog(leakLogFile, options.leakThreshold)
+      processLeakLog(self.leakLogFile, options.leakThreshold)
       self.automation.log.info("\nREFTEST INFO | runreftest.py | Running tests: end.")
     finally:
-      if profileDir:
-        shutil.rmtree(profileDir)
+      self.cleanup(profileDir)
     return status
 
   def copyExtraFilesToProfile(self, options, profileDir):
@@ -155,41 +174,86 @@ class RefTest(object):
       else:
         shutil.copy(abspath, dest)
 
+  def installExtensionsToProfile(self, options, profileDir):
+    "Install the specified extensions on the command line to the testing profile."
+    for f in options.extensionsToInstall:
+      abspath = self.getFullPath(f)
+      extensionID = f[:f.rfind(".")]
+      self.automation.installExtension(abspath, profileDir, extensionID)
 
-def main():
-  automation = Automation()
-  parser = OptionParser()
-  reftest = RefTest(automation)
 
-  # we want to pass down everything from automation.__all__
-  addCommonOptions(parser, 
-                   defaults=dict(zip(automation.__all__, 
-                            [getattr(automation, x) for x in automation.__all__])))
-  automation.addCommonOptions(parser)
-  parser.add_option("--appname",
+class ReftestOptions(OptionParser):
+
+  def __init__(self, automation):
+    self._automation = automation
+    OptionParser.__init__(self)
+    defaults = {}
+
+    # we want to pass down everything from automation.__all__
+    addCommonOptions(self, 
+                     defaults=dict(zip(self._automation.__all__, 
+                            [getattr(self._automation, x) for x in self._automation.__all__])))
+    self._automation.addCommonOptions(self)
+    self.add_option("--appname",
                     action = "store", type = "string", dest = "app",
                     default = os.path.join(SCRIPT_DIRECTORY, automation.DEFAULT_APP),
                     help = "absolute path to application, overriding default")
-  parser.add_option("--extra-profile-file",
+    self.add_option("--extra-profile-file",
                     action = "append", dest = "extraProfileFiles",
                     default = [],
                     help = "copy specified files/dirs to testing profile")
-  parser.add_option("--timeout",              
+    self.add_option("--timeout",              
                     action = "store", dest = "timeout", type = "int", 
                     default = 5 * 60, # 5 minutes per bug 479518
                     help = "reftest will timeout in specified number of seconds. [default %default s].")
-  parser.add_option("--leak-threshold",
+    self.add_option("--leak-threshold",
                     action = "store", type = "int", dest = "leakThreshold",
                     default = 0,
                     help = "fail if the number of bytes leaked through "
                            "refcounted objects (or bytes in classes with "
                            "MOZ_COUNT_CTOR and MOZ_COUNT_DTOR) is greater "
                            "than the given number")
-  parser.add_option("--utility-path",
+    self.add_option("--utility-path",
                     action = "store", type = "string", dest = "utilityPath",
-                    default = automation.DIST_BIN,
+                    default = self._automation.DIST_BIN,
                     help = "absolute path to directory containing utility "
                            "programs (xpcshell, ssltunnel, certutil)")
+    defaults["utilityPath"] = self._automation.DIST_BIN
+
+    self.add_option("--total-chunks",
+                    type = "int", dest = "totalChunks",
+                    help = "how many chunks to split the tests up into")
+    defaults["totalChunks"] = None
+
+    self.add_option("--this-chunk",
+                    type = "int", dest = "thisChunk",
+                    help = "which chunk to run between 1 and --total-chunks")
+    defaults["thisChunk"] = None
+
+    self.add_option("--log-file",
+                    action = "store", type = "string", dest = "logFile",
+                    default = None,
+                    help = "file to log output to in addition to stdout")
+    defaults["logFile"] = None
+ 
+    self.add_option("--skip-slow-tests",
+                    dest = "skipSlowTests", action = "store_true",
+                    help = "skip tests marked as slow when running")
+    defaults["skipSlowTests"] = False
+
+    self.add_option("--install-extension",
+                    action = "append", dest = "extensionsToInstall",
+                    help = "install the specified extension in the testing profile."
+                           "The extension file's name should be <id>.xpi where <id> is"
+                           "the extension's id as indicated in its install.rdf.")
+    defaults["extensionsToInstall"] = []
+
+    self.set_defaults(**defaults)
+
+def main():
+  automation = Automation()
+  parser = ReftestOptions(automation)
+  reftest = RefTest(automation)
 
   options, args = parser.parse_args()
   if len(args) != 1:
@@ -209,10 +273,22 @@ Are you executing $objdir/_tests/reftest/runreftest.py?""" \
     # allow relative paths
     options.xrePath = reftest.getFullPath(options.xrePath)
 
-  if options.symbolsPath:
+  if options.symbolsPath and not isURL(options.symbolsPath):
     options.symbolsPath = reftest.getFullPath(options.symbolsPath)
   options.utilityPath = reftest.getFullPath(options.utilityPath)
 
+  if options.totalChunks is not None and options.thisChunk is None:
+    print "thisChunk must be specified when totalChunks is specified"
+    sys.exit(1)
+
+  if options.totalChunks:
+    if not 1 <= options.thisChunk <= options.totalChunks:
+      print "thisChunk must be between 1 and totalChunks"
+      sys.exit(1)
+  
+  if options.logFile:
+    options.logFile = reftest.getFullPath(options.logFile)
+
   sys.exit(reftest.runTests(args[0], options))
   
 if __name__ == "__main__":
