@@ -23,6 +23,7 @@
  *   Brett Wilson <brettw@gmail.com>
  *   Stuart Parmenter <pavlov@pavlov.net>
  *   Justin Dolske <dolske@mozilla.com>
+ *   Glenn Randers-Pehrson <glennrp@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -45,15 +46,16 @@
 #include "nsString.h"
 #include "nsStreamUtils.h"
 
-// Input streams that do not implement nsIAsyncInputStream should be threadsafe
-// so that they may be used with nsIInputStreamPump and nsIInputStreamChannel,
-// which read such a stream on a background thread.
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsPNGEncoder, imgIEncoder, nsIInputStream)
+NS_IMPL_THREADSAFE_ISUPPORTS3(nsPNGEncoder, imgIEncoder, nsIInputStream, nsIAsyncInputStream)
 
 nsPNGEncoder::nsPNGEncoder() : mPNG(nsnull), mPNGinfo(nsnull),
                                mIsAnimation(PR_FALSE),
+                               mFinished(PR_FALSE),
                                mImageBuffer(nsnull), mImageBufferSize(0),
-                               mImageBufferUsed(0), mImageBufferReadPoint(0)
+                               mImageBufferUsed(0), mImageBufferReadPoint(0),
+                               mCallback(nsnull),
+                               mCallbackTarget(nsnull), mNotifyThreshold(0),
+                               mMonitor("PNG Encoder Monitor")
 {
 }
 
@@ -77,12 +79,13 @@ nsPNGEncoder::~nsPNGEncoder()
 //    See also modules/libimg/png/libpng.txt
 
 NS_IMETHODIMP nsPNGEncoder::InitFromData(const PRUint8* aData,
-                                          PRUint32 aLength, // (unused, req'd by JS)
-                                          PRUint32 aWidth,
-                                          PRUint32 aHeight,
-                                          PRUint32 aStride,
-                                          PRUint32 aInputFormat,
-                                          const nsAString& aOutputOptions)
+                                         PRUint32 aLength, // (unused,
+                                                           // req'd by JS)
+                                         PRUint32 aWidth,
+                                         PRUint32 aHeight,
+                                         PRUint32 aStride,
+                                         PRUint32 aInputFormat,
+                                         const nsAString& aOutputOptions)
 {
   nsresult rv;
 
@@ -90,7 +93,8 @@ NS_IMETHODIMP nsPNGEncoder::InitFromData(const PRUint8* aData,
   if (!NS_SUCCEEDED(rv))
     return rv;
 
-  rv = AddImageFrame(aData, aLength, aWidth, aHeight, aStride, aInputFormat, aOutputOptions);
+  rv = AddImageFrame(aData, aLength, aWidth, aHeight, aStride,
+                     aInputFormat, aOutputOptions);
   if (!NS_SUCCEEDED(rv))
     return rv;
 
@@ -127,15 +131,18 @@ NS_IMETHODIMP nsPNGEncoder::StartImageEncode(PRUint32 aWidth,
   nsresult rv = ParseOptions(aOutputOptions, &useTransparency, &skipFirstFrame,
                              &numFrames, &numPlays, nsnull, nsnull,
                              nsnull, nsnull, nsnull);
-  if (rv != NS_OK) { return rv; }
+  if (rv != NS_OK)
+    return rv;
 
-  if (numFrames > 1) {
+#ifdef PNG_APNG_SUPPORTED
+  if (numFrames > 1)
     mIsAnimation = PR_TRUE;
-  }
+
+#endif
 
   // initialize
   mPNG = png_create_write_struct(PNG_LIBPNG_VER_STRING,
-                                 png_voidp_NULL,
+                                 nsnull,
                                  ErrorCallback,
                                  ErrorCallback);
   if (! mPNG)
@@ -167,12 +174,13 @@ NS_IMETHODIMP nsPNGEncoder::StartImageEncode(PRUint32 aWidth,
   mImageBufferUsed = 0;
 
   // set our callback for libpng to give us the data
-  png_set_write_fn(mPNG, this, WriteCallback, NULL);
+  png_set_write_fn(mPNG, this, WriteCallback, nsnull);
 
   // include alpha?
   int colorType;
   if ((aInputFormat == INPUT_FORMAT_HOSTARGB ||
-       aInputFormat == INPUT_FORMAT_RGBA)  &&  useTransparency)
+       aInputFormat == INPUT_FORMAT_RGBA)  &&
+       useTransparency)
     colorType = PNG_COLOR_TYPE_RGB_ALPHA;
   else
     colorType = PNG_COLOR_TYPE_RGB;
@@ -181,10 +189,12 @@ NS_IMETHODIMP nsPNGEncoder::StartImageEncode(PRUint32 aWidth,
                PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
                PNG_FILTER_TYPE_DEFAULT);
 
+#ifdef PNG_APNG_SUPPORTED
   if (mIsAnimation) {
     png_set_first_frame_is_hidden(mPNG, mPNGinfo, skipFirstFrame);
     png_set_acTL(mPNG, mPNGinfo, numFrames, numPlays);
   }
+#endif
 
   // XXX: support PLTE, gAMA, tRNS, bKGD?
 
@@ -195,7 +205,8 @@ NS_IMETHODIMP nsPNGEncoder::StartImageEncode(PRUint32 aWidth,
 
 
 NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
-                                          PRUint32 aLength, // (unused, req'd by JS)
+                                          PRUint32 aLength, // (unused,
+                                                            // req'd by JS)
                                           PRUint32 aWidth,
                                           PRUint32 aHeight,
                                           PRUint32 aStride,
@@ -204,8 +215,13 @@ NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
 {
   PRBool useTransparency= PR_TRUE;
   PRUint32 delay_ms = 500;
+#ifdef PNG_APNG_SUPPORTED
   PRUint32 dispose_op = PNG_DISPOSE_OP_NONE;
   PRUint32 blend_op = PNG_BLEND_OP_SOURCE;
+#else
+  PRUint32 dispose_op;
+  PRUint32 blend_op;
+#endif
   PRUint32 x_offset = 0, y_offset = 0;
 
   // must be initialized
@@ -232,21 +248,26 @@ NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
   nsresult rv = ParseOptions(aFrameOptions, &useTransparency, nsnull,
                              nsnull, nsnull, &dispose_op, &blend_op,
                              &delay_ms, &x_offset, &y_offset);
-  if (rv != NS_OK) { return rv; }
+  if (rv != NS_OK)
+    return rv;
 
+#ifdef PNG_APNG_SUPPORTED
   if (mIsAnimation) {
     // XXX the row pointers arg (#3) is unused, can it be removed?
     png_write_frame_head(mPNG, mPNGinfo, nsnull,
                          aWidth, aHeight, x_offset, y_offset,
                          delay_ms, 1000, dispose_op, blend_op);
   }
+#endif
 
-  // Stride is the padded width of each row, so it better be longer (I'm afraid
-  // people will not understand what stride means, so check it well)
+  // Stride is the padded width of each row, so it better be longer 
+  // (I'm afraid people will not understand what stride means, so
+  // check it well)
   if ((aInputFormat == INPUT_FORMAT_RGB &&
-       aStride < aWidth * 3) ||
-      ((aInputFormat == INPUT_FORMAT_RGBA || aInputFormat == INPUT_FORMAT_HOSTARGB) &&
-       aStride < aWidth * 4)) {
+      aStride < aWidth * 3) ||
+      ((aInputFormat == INPUT_FORMAT_RGBA ||
+      aInputFormat == INPUT_FORMAT_HOSTARGB) &&
+      aStride < aWidth * 4)) {
     NS_WARNING("Invalid stride for InitFromData/AddImageFrame");
     return NS_ERROR_INVALID_ARG;
   }
@@ -254,7 +275,8 @@ NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
   // write each row: if we add more input formats, we may want to
   // generalize the conversions
   if (aInputFormat == INPUT_FORMAT_HOSTARGB) {
-    // PNG requires RGBA with post-multiplied alpha, so we need to convert
+    // PNG requires RGBA with post-multiplied alpha, so we need to
+    // convert
     PRUint8* row = new PRUint8[aWidth * 4];
     for (PRUint32 y = 0; y < aHeight; y ++) {
       ConvertHostARGBRow(&aData[y * aStride], row, aWidth, useTransparency);
@@ -283,9 +305,11 @@ NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
     return NS_ERROR_INVALID_ARG;
   }
 
+#ifdef PNG_APNG_SUPPORTED
   if (mIsAnimation) {
     png_write_frame_tail(mPNG, mPNGinfo);
   }
+#endif
 
   return NS_OK;
 }
@@ -297,7 +321,8 @@ NS_IMETHODIMP nsPNGEncoder::EndImageEncode()
   if (mImageBuffer == nsnull)
     return NS_ERROR_NOT_INITIALIZED;
 
-  // EndImageEncode has already been called, or some error occurred earlier
+  // EndImageEncode has already been called, or some error
+  // occurred earlier
   if (!mPNG)
     return NS_BASE_STREAM_CLOSED;
 
@@ -310,6 +335,9 @@ NS_IMETHODIMP nsPNGEncoder::EndImageEncode()
   png_write_end(mPNG, mPNGinfo);
   png_destroy_write_struct(&mPNG, &mPNGinfo);
 
+  mFinished = PR_TRUE;
+  NotifyListener();
+
   // if output callback can't get enough memory, it will free our buffer
   if (!mImageBuffer)
     return NS_ERROR_OUT_OF_MEMORY;
@@ -330,6 +358,7 @@ nsPNGEncoder::ParseOptions(const nsAString& aOptions,
                            PRUint32* offsetX,
                            PRUint32* offsetY)
 {
+#ifdef PNG_APNG_SUPPORTED
   // Make a copy of aOptions, because strtok() will modify it.
   nsCAutoString optionsCopy;
   optionsCopy.Assign(NS_ConvertUTF16toUTF8(aOptions));
@@ -338,16 +367,22 @@ nsPNGEncoder::ParseOptions(const nsAString& aOptions,
   while (char* token = nsCRT::strtok(options, ";", &options)) {
     // If there's an '=' character, split the token around it.
     char* equals = token, *value = nsnull;
-    while(*equals != '=' && *equals) { ++equals; }
-    if (*equals == '=') { value = equals + 1; }
+    while(*equals != '=' && *equals) {
+      ++equals;
+    }
+    if (*equals == '=')
+      value = equals + 1;
 
-    if (value) { *equals = '\0'; } // temporary null
+    if (value)
+      *equals = '\0'; // temporary null
 
     // transparency=[yes|no|none]
     if (nsCRT::strcmp(token, "transparency") == 0 && useTransparency) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
-      if (nsCRT::strcmp(value, "none") == 0 || nsCRT::strcmp(value, "no") == 0) {
+      if (nsCRT::strcmp(value, "none") == 0 ||
+          nsCRT::strcmp(value, "no") == 0) {
         *useTransparency = PR_FALSE;
       } else if (nsCRT::strcmp(value, "yes") == 0) {
         *useTransparency = PR_TRUE;
@@ -356,8 +391,10 @@ nsPNGEncoder::ParseOptions(const nsAString& aOptions,
       }
 
     // skipfirstframe=[yes|no]
-    } else if (nsCRT::strcmp(token, "skipfirstframe") == 0 && skipFirstFrame) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+    } else if (nsCRT::strcmp(token, "skipfirstframe") == 0 &&
+               skipFirstFrame) {
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
       if (nsCRT::strcmp(value, "no") == 0) {
         *skipFirstFrame = PR_FALSE;
@@ -369,23 +406,31 @@ nsPNGEncoder::ParseOptions(const nsAString& aOptions,
 
     // frames=#
     } else if (nsCRT::strcmp(token, "frames") == 0 && numFrames) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
-      if (PR_sscanf(value, "%u", numFrames) != 1) { return NS_ERROR_INVALID_ARG; }
+      if (PR_sscanf(value, "%u", numFrames) != 1) {
+        return NS_ERROR_INVALID_ARG;
+      }
 
       // frames=0 is nonsense.
-      if (*numFrames == 0) { return NS_ERROR_INVALID_ARG; }
+      if (*numFrames == 0)
+        return NS_ERROR_INVALID_ARG;
 
     // plays=#
     } else if (nsCRT::strcmp(token, "plays") == 0 && numPlays) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
-      // plays=0 to loop forever, otherwise play sequence specified number of times
-      if (PR_sscanf(value, "%u", numPlays) != 1) { return NS_ERROR_INVALID_ARG; }
+      // plays=0 to loop forever, otherwise play sequence specified
+      // number of times
+      if (PR_sscanf(value, "%u", numPlays) != 1)
+        return NS_ERROR_INVALID_ARG;
 
     // dispose=[none|background|previous]
     } else if (nsCRT::strcmp(token, "dispose") == 0 && frameDispose) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
       if (nsCRT::strcmp(value, "none") == 0) {
         *frameDispose = PNG_DISPOSE_OP_NONE;
@@ -399,7 +444,8 @@ nsPNGEncoder::ParseOptions(const nsAString& aOptions,
 
     // blend=[source|over]
     } else if (nsCRT::strcmp(token, "blend") == 0 && frameBlend) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
       if (nsCRT::strcmp(value, "source") == 0) {
         *frameBlend = PNG_BLEND_OP_SOURCE;
@@ -411,30 +457,37 @@ nsPNGEncoder::ParseOptions(const nsAString& aOptions,
 
     // delay=# (in ms)
     } else if (nsCRT::strcmp(token, "delay") == 0 && frameDelay) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
-      if (PR_sscanf(value, "%u", frameDelay) != 1) { return NS_ERROR_INVALID_ARG; }
+      if (PR_sscanf(value, "%u", frameDelay) != 1)
+        return NS_ERROR_INVALID_ARG;
 
     // xoffset=#
     } else if (nsCRT::strcmp(token, "xoffset") == 0 && offsetX) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
-      if (PR_sscanf(value, "%u", offsetX) != 1) { return NS_ERROR_INVALID_ARG; }
+      if (PR_sscanf(value, "%u", offsetX) != 1)
+        return NS_ERROR_INVALID_ARG;
 
     // yoffset=#
     } else if (nsCRT::strcmp(token, "yoffset") == 0 && offsetY) {
-      if (!value) { return NS_ERROR_INVALID_ARG; }
+      if (!value)
+        return NS_ERROR_INVALID_ARG;
 
-      if (PR_sscanf(value, "%u", offsetY) != 1) { return NS_ERROR_INVALID_ARG; }
+      if (PR_sscanf(value, "%u", offsetY) != 1)
+        return NS_ERROR_INVALID_ARG;
 
     // unknown token name
-    } else {
+    } else
       return NS_ERROR_INVALID_ARG;
-    }
 
-    if (value) { *equals = '='; } // restore '=' so strtok doesn't get lost
+    if (value)
+      *equals = '='; // restore '=' so strtok doesn't get lost
   }
 
+#endif
   return NS_OK;
 }
 
@@ -462,29 +515,36 @@ NS_IMETHODIMP nsPNGEncoder::Available(PRUint32 *_retval)
   return NS_OK;
 }
 
-/* [noscript] unsigned long read (in charPtr aBuf, in unsigned long aCount); */
+/* [noscript] unsigned long read (in charPtr aBuf,
+                                  in unsigned long aCount); */
 NS_IMETHODIMP nsPNGEncoder::Read(char * aBuf, PRUint32 aCount,
                                  PRUint32 *_retval)
 {
   return ReadSegments(NS_CopySegmentToBuffer, aBuf, aCount, _retval);
 }
 
-/* [noscript] unsigned long readSegments (in nsWriteSegmentFun aWriter, in voidPtr aClosure, in unsigned long aCount); */
+/* [noscript] unsigned long readSegments (in nsWriteSegmentFun aWriter,
+                                          in voidPtr aClosure,
+                                          in unsigned long aCount); */
 NS_IMETHODIMP nsPNGEncoder::ReadSegments(nsWriteSegmentFun aWriter,
                                          void *aClosure, PRUint32 aCount,
                                          PRUint32 *_retval)
 {
+  // Avoid another thread reallocing the buffer underneath us
+  mozilla::MonitorAutoEnter autoEnter(mMonitor);
+
   PRUint32 maxCount = mImageBufferUsed - mImageBufferReadPoint;
   if (maxCount == 0) {
     *_retval = 0;
-    return NS_OK;
+    return mFinished ? NS_OK : NS_BASE_STREAM_WOULD_BLOCK;
   }
 
   if (aCount > maxCount)
     aCount = maxCount;
-  nsresult rv = aWriter(this, aClosure,
-                        reinterpret_cast<const char*>(mImageBuffer+mImageBufferReadPoint),
-                        0, aCount, _retval);
+  nsresult rv =
+      aWriter(this, aClosure,
+              reinterpret_cast<const char*>(mImageBuffer+mImageBufferReadPoint),
+              0, aCount, _retval);
   if (NS_SUCCEEDED(rv)) {
     NS_ASSERTION(*_retval <= aCount, "bad write count");
     mImageBufferReadPoint += *_retval;
@@ -497,10 +557,42 @@ NS_IMETHODIMP nsPNGEncoder::ReadSegments(nsWriteSegmentFun aWriter,
 /* boolean isNonBlocking (); */
 NS_IMETHODIMP nsPNGEncoder::IsNonBlocking(PRBool *_retval)
 {
-  *_retval = PR_FALSE;  // We don't implement nsIAsyncInputStream
+  *_retval = PR_TRUE;
   return NS_OK;
 }
 
+NS_IMETHODIMP nsPNGEncoder::AsyncWait(nsIInputStreamCallback *aCallback,
+                                      PRUint32 aFlags,
+                                      PRUint32 aRequestedCount,
+                                      nsIEventTarget *aTarget)
+{
+  if (aFlags != 0)
+    return NS_ERROR_NOT_IMPLEMENTED;
+
+  if (mCallback || mCallbackTarget)
+    return NS_ERROR_UNEXPECTED;
+
+  mCallbackTarget = aTarget;
+  // 0 means "any number of bytes except 0"
+  mNotifyThreshold = aRequestedCount;
+  if (!aRequestedCount)
+    mNotifyThreshold = 1024; // We don't want to notify incessantly
+
+  // We set the callback absolutely last, because NotifyListener uses it to
+  // determine if someone needs to be notified.  If we don't set it last,
+  // NotifyListener might try to fire off a notification to a null target
+  // which will generally cause non-threadsafe objects to be used off the main thread
+  mCallback = aCallback;
+
+  // What we are being asked for may be present already
+  NotifyListener();
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPNGEncoder::CloseWithStatus(nsresult aStatus)
+{
+  return Close();
+}
 
 // nsPNGEncoder::ConvertHostARGBRow
 //
@@ -511,7 +603,8 @@ NS_IMETHODIMP nsPNGEncoder::IsNonBlocking(PRBool *_retval)
 
 void
 nsPNGEncoder::ConvertHostARGBRow(const PRUint8* aSrc, PRUint8* aDest,
-                                  PRUint32 aPixelWidth, PRBool aUseTransparency)
+                                 PRUint32 aPixelWidth,
+                                 PRBool aUseTransparency)
 {
   PRUint32 pixelStride = aUseTransparency ? 4 : 3;
   for (PRUint32 x = 0; x < aPixelWidth; x ++) {
@@ -553,10 +646,12 @@ nsPNGEncoder::StripAlpha(const PRUint8* aSrc, PRUint8* aDest,
 // nsPNGEncoder::ErrorCallback
 
 void // static
-nsPNGEncoder::ErrorCallback(png_structp png_ptr, png_const_charp warning_msg)
+nsPNGEncoder::ErrorCallback(png_structp png_ptr,
+                            png_const_charp warning_msg)
 {
 #ifdef DEBUG
-	// XXX: these messages are probably useful callers... use nsIConsoleService?
+	// XXX: these messages are probably useful callers...
+        // use nsIConsoleService?
 	PR_fprintf(PR_STDERR, "PNG Encoder: %s\n", warning_msg);;
 #endif
 }
@@ -565,13 +660,18 @@ nsPNGEncoder::ErrorCallback(png_structp png_ptr, png_const_charp warning_msg)
 // nsPNGEncoder::WriteCallback
 
 void // static
-nsPNGEncoder::WriteCallback(png_structp png, png_bytep data, png_size_t size)
+nsPNGEncoder::WriteCallback(png_structp png, png_bytep data,
+                            png_size_t size)
 {
   nsPNGEncoder* that = static_cast<nsPNGEncoder*>(png_get_io_ptr(png));
   if (! that->mImageBuffer)
     return;
 
   if (that->mImageBufferUsed + size > that->mImageBufferSize) {
+    // When we're reallocing the buffer we need to take the lock to ensure
+    // that nobody is trying to read from the buffer we are destroying
+    mozilla::MonitorAutoEnter autoEnter(that->mMonitor);
+
     // expand buffer, just double each time
     that->mImageBufferSize *= 2;
     PRUint8* newBuf = (PRUint8*)PR_Realloc(that->mImageBuffer,
@@ -587,4 +687,37 @@ nsPNGEncoder::WriteCallback(png_structp png, png_bytep data, png_size_t size)
   }
   memcpy(&that->mImageBuffer[that->mImageBufferUsed], data, size);
   that->mImageBufferUsed += size;
+  that->NotifyListener();
+}
+
+void
+nsPNGEncoder::NotifyListener()
+{
+  // We might call this function on multiple threads (any threads that call
+  // AsyncWait and any that do encoding) so we lock to avoid notifying the
+  // listener twice about the same data (which generally leads to a truncated
+  // image).
+  mozilla::MonitorAutoEnter autoEnter(mMonitor);
+
+  if (mCallback &&
+      (mImageBufferUsed - mImageBufferReadPoint >= mNotifyThreshold ||
+       mFinished)) {
+    nsCOMPtr<nsIInputStreamCallback> callback;
+    if (mCallbackTarget) {
+      NS_NewInputStreamReadyEvent(getter_AddRefs(callback),
+                                  mCallback,
+                                  mCallbackTarget);
+    } else {
+      callback = mCallback;
+    }
+
+    NS_ASSERTION(callback, "Shouldn't fail to make the callback");
+    // Null the callback first because OnInputStreamReady could reenter
+    // AsyncWait
+    mCallback = nsnull;
+    mCallbackTarget = nsnull;
+    mNotifyThreshold = 0;
+
+    callback->OnInputStreamReady(this);
+  }
 }
