@@ -261,8 +261,10 @@ static void FreeArray(nsDiscriminatedUnion* data)
 
 static nsresult CloneArray(PRUint16 inType, const nsIID* inIID,
                            PRUint32 inCount, void* inValue,
-                           PRUint16* outType, nsIID* outIID,
-                           PRUint32* outCount, void** outValue)
+                           PRUint16* outType NS_OUTPARAM,
+                           nsIID* outIID NS_OUTPARAM,
+                           PRUint32* outCount NS_OUTPARAM,
+                           void** outValue)
 {
     NS_ASSERTION(inCount, "bad param");
     NS_ASSERTION(inValue, "bad param");
@@ -1884,6 +1886,13 @@ NS_IMETHODIMP nsVariant::GetAsISupports(nsISupports **_retval)
     return nsVariant::ConvertToISupports(mData, _retval);
 }
 
+/* jsval getAsJSVal() */
+NS_IMETHODIMP nsVariant::GetAsJSVal(jsval *_retval)
+{
+    // Can only get the jsval from an XPCVariant.
+    return NS_ERROR_CANNOT_CONVERT_DATA;
+}
+
 /* void getAsInterface (out nsIIDPtr iid, [iid_is (iid), retval] out nsQIResult iface); */
 NS_IMETHODIMP nsVariant::GetAsInterface(nsIID * *iid, void * *iface)
 {
